@@ -1,25 +1,33 @@
-/* Copyright (c) 2000-2008 MySQL AB
-
+/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+#include "sql_priv.h"
+#include "my_global.h"                          // HAVE_*
 
-#include "mysql_priv.h"
 #ifdef HAVE_QUERY_CACHE
 #include <mysql.h>
 #include "emb_qcache.h"
 #include "embedded_priv.h"
+#include "sql_class.h"                          // THD
 
 void Querycache_stream::store_uchar(uchar c)
 {
@@ -190,12 +198,12 @@ uint Querycache_stream::load_int()
     cur_data+= 4;
     return result;
   }
-  char buf[4];
+  char buf[4], *buf_p= buf;
   memcpy(buf, cur_data, rest_len);
   use_next_block(FALSE);
   memcpy(buf+rest_len, cur_data, 4-rest_len);
   cur_data+= 4-rest_len;
-  result= uint4korr(buf);
+  result= uint4korr(buf_p);
   return result;
 }
 
@@ -485,7 +493,8 @@ int emb_load_querycache_result(THD *thd, Querycache_stream *src)
   *prev_row= NULL;
   data->embedded_info->prev_ptr= prev_row;
 return_ok:
-  net_send_eof(thd, thd->server_status, thd->total_warn_count);
+  net_send_eof(thd, thd->server_status,
+               thd->get_stmt_da()->current_statement_warn_count());
   DBUG_RETURN(0);
 err:
   DBUG_RETURN(1);
