@@ -45,7 +45,7 @@ struct NDB_SCHEMA_OBJECT {
   pthread_mutex_t mutex; //Protects NDB_SCHEMA_OBJ and 'cond'
   pthread_cond_t cond;   //Signal/wait slock_bitmap changes
   char *key;
-  uint key_length;
+  size_t key_length;
   uint use_count;
   MY_BITMAP slock_bitmap;
   uint32 slock[256/32]; // 256 bits for lock status of table
@@ -54,10 +54,10 @@ struct NDB_SCHEMA_OBJECT {
 
 public:
   // Check all Clients if any should wakeup due to new participant status
-  static void check_waiters();
+  static void check_waiters(const MY_BITMAP &new_participants);
 
 private:
-  void check_waiter();
+  void check_waiter(const MY_BITMAP &new_participants);
 };
 
 NDB_SCHEMA_OBJECT *ndb_get_schema_object(const char *key,
