@@ -1,15 +1,21 @@
 /*
-   Copyright (c) 2003-2007 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -21,43 +27,20 @@
 
 #include "SignalData.hpp"
 
-/**
- * AlterTable
- *
- * This signal is sent by API to DICT/TRIX
- * as a request to alter a secondary index
- * and then from TRIX to TRIX(n) and TRIX to TC.
- */
-class AlterTableReq {
-  /**
-   * Sender(s)
-   */
-  // API
-  
-  /**
-   * Sender(s) / Reciver(s)
-   */
-  friend class NdbTableImpl;
-  friend class NdbEventOperationImpl;
-  friend class NdbDictInterface;
-  friend class Dbdict;
-  friend class Dbtup;
-  friend class Suma;
+#define JAM_FILE_ID 112
 
-  /**
-   * For printing
-   */
-  friend bool printALTER_TABLE_REQ(FILE*, const Uint32*, Uint32, Uint16);
 
-public:
-  STATIC_CONST( SignalLength = 5 );
+struct AlterTableReq {
+  STATIC_CONST( SignalLength = 8 );
   
-private:
-  Uint32 senderData;
-  Uint32 senderRef;
-  Uint32 changeMask;
+  Uint32 clientRef;
+  Uint32 clientData;
+  Uint32 transId;
+  Uint32 transKey;
+  Uint32 requestInfo;
   Uint32 tableId;
   Uint32 tableVersion;
+  Uint32 changeMask;
 
   SECTION( DICT_TAB_INFO = 0 );
 
@@ -73,10 +56,12 @@ private:
   t = Changed tablespace name array
   s = Changed tablespace id array
   a = Add attribute
+  f = Add fragment(s)
+  r = Reorg fragment(s)
 
            1111111111222222222233
  01234567890123456789012345678901
- nfdrtsa-------------------------
+ nfdrtsafr-----------------------
 */
 #define NAME_SHIFT        (0)
 #define FRM_SHIFT         (1)
@@ -85,6 +70,12 @@ private:
 #define TS_NAME_SHIFT     (4)
 #define TS_SHIFT          (5)
 #define ADD_ATTR_SHIFT    (6)
+#define ADD_FRAG_SHIFT    (7)
+#define REORG_FRAG_SHIFT  (8)
+#define REORG_COMMIT_SHIFT   (9)
+#define REORG_COMPLETE_SHIFT (10)
+#define REORG_SUMA_ENABLE (11)
+#define REORG_SUMA_FILTER (12)
 
  /**
    * Getters and setters
@@ -103,6 +94,26 @@ private:
   static void setTsFlag(UintR &  changeMask, Uint32 tsFlg);
   static Uint8 getAddAttrFlag(const UintR & changeMask);
   static void setAddAttrFlag(UintR &  changeMask, Uint32 tsFlg);
+  static Uint8 getAddFragFlag(const UintR & changeMask);
+  static void setAddFragFlag(UintR &  changeMask, Uint32 tsFlg);
+  static Uint8 getReorgFragFlag(const UintR & changeMask);
+  static void setReorgFragFlag(UintR &  changeMask, Uint32 tsFlg);
+  static Uint8 getReorgCommitFlag(const UintR & changeMask);
+  static void setReorgCommitFlag(UintR &  changeMask, Uint32 tsFlg);
+  static Uint8 getReorgCompleteFlag(const UintR & changeMask);
+  static void setReorgCompleteFlag(UintR &  changeMask, Uint32 tsFlg);
+  static Uint8 getReorgSumaEnableFlag(const UintR & changeMask);
+  static void setReorgSumaEnableFlag(UintR &  changeMask, Uint32 tsFlg);
+  static Uint8 getReorgSumaFilterFlag(const UintR & changeMask);
+  static void setReorgSumaFilterFlag(UintR &  changeMask, Uint32 tsFlg);
+
+  static bool getReorgSubOp(const UintR & changeMask){
+    return
+      getReorgCommitFlag(changeMask) ||
+      getReorgCompleteFlag(changeMask) ||
+      getReorgSumaEnableFlag(changeMask) ||
+      getReorgSumaFilterFlag(changeMask);
+  }
 };
 
 inline
@@ -189,26 +200,95 @@ AlterTableReq::setAddAttrFlag(UintR & changeMask, Uint32 addAttrFlg){
   changeMask |= (addAttrFlg << ADD_ATTR_SHIFT);
 }
 
+inline
+Uint8
+AlterTableReq::getAddFragFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> ADD_FRAG_SHIFT) & 1);
+}
 
-class AlterTableRef {
-  /**
-   * Sender(s)
-   */
-  friend class Dbdict;
-  
-  /**
-   * Sender(s) / Reciver(s)
-   */
-  friend class Ndbcntr;
-  friend class NdbDictInterface;
-  
-  /**
-   * For printing
-   */
-  friend bool printALTER_TABLE_REF(FILE *, const Uint32 *, Uint32, Uint16);
-  
-public:
-  STATIC_CONST( SignalLength = 7 );
+inline
+void
+AlterTableReq::setAddFragFlag(UintR & changeMask, Uint32 addAttrFlg){
+  changeMask |= (addAttrFlg << ADD_FRAG_SHIFT);
+}
+
+inline
+Uint8
+AlterTableReq::getReorgFragFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> REORG_FRAG_SHIFT) & 1);
+}
+
+inline
+void
+AlterTableReq::setReorgFragFlag(UintR & changeMask, Uint32 reorgAttrFlg){
+  changeMask |= (reorgAttrFlg << REORG_FRAG_SHIFT);
+}
+
+inline
+Uint8
+AlterTableReq::getReorgCommitFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> REORG_COMMIT_SHIFT) & 1);
+}
+
+inline
+void
+AlterTableReq::setReorgCommitFlag(UintR & changeMask, Uint32 reorgAttrFlg){
+  changeMask |= (reorgAttrFlg << REORG_COMMIT_SHIFT);
+}
+
+
+inline
+Uint8
+AlterTableReq::getReorgCompleteFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> REORG_COMPLETE_SHIFT) & 1);
+}
+
+inline
+void
+AlterTableReq::setReorgCompleteFlag(UintR & changeMask, Uint32 reorgAttrFlg){
+  changeMask |= (reorgAttrFlg << REORG_COMPLETE_SHIFT);
+}
+
+inline
+Uint8
+AlterTableReq::getReorgSumaEnableFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> REORG_SUMA_ENABLE) & 1);
+}
+
+inline
+void
+AlterTableReq::setReorgSumaEnableFlag(UintR & changeMask, Uint32 reorgAttrFlg){
+  changeMask |= (reorgAttrFlg << REORG_SUMA_ENABLE);
+}
+
+inline
+Uint8
+AlterTableReq::getReorgSumaFilterFlag(const UintR & changeMask){
+  return (Uint8)((changeMask >> REORG_SUMA_FILTER) & 1);
+}
+
+inline
+void
+AlterTableReq::setReorgSumaFilterFlag(UintR & changeMask, Uint32 reorgAttrFlg){
+  changeMask |= (reorgAttrFlg << REORG_SUMA_FILTER);
+}
+
+struct AlterTableConf {
+  STATIC_CONST( SignalLength = 6 );
+
+  Uint32 senderRef;
+  union {
+    Uint32 clientData;
+    Uint32 senderData;
+  };
+  Uint32 transId;
+  Uint32 tableId;
+  Uint32 tableVersion;
+  Uint32 newTableVersion;
+};
+
+struct AlterTableRef {
+  STATIC_CONST( SignalLength = 9 );
 
   enum ErrorCode {
     NoError = 0,
@@ -233,52 +313,22 @@ public:
     UnsupportedChange = 741,
     BackupInProgress = 762,
     IncompatibleVersions = 763,
-    SingleUser = 299
+    SingleUser = 299,
+    TableDefinitionTooBig = 793
   };
 
-private:
-  Uint32 senderData;
   Uint32 senderRef;
-  Uint32 masterNodeId;
+  union {
+    Uint32 clientData;
+    Uint32 senderData;
+  };
+  Uint32 transId;
   Uint32 errorCode;
   Uint32 errorLine; 
+  Uint32 errorNodeId;
+  Uint32 masterNodeId;
+  Uint32 errorStatus;
   Uint32 errorKey;
-  Uint32 status;
-
-public:
-  Uint32 getErrorCode() const {
-    return errorCode;
-  }
-  Uint32 getErrorLine() const {
-    return errorLine;
-  }
-};
-
-class AlterTableConf {
-  /**
-   * Sender(s)
-   */
-  friend class Dbdict;
-  
-  /**
-   * Sender(s) / Reciver(s)
-   */
-  friend class Ndbcntr;
-  friend class NdbDictInterface;
-  
-  /**
-   * For printing
-   */
-  friend bool printALTER_TABLE_CONF(FILE *, const Uint32 *, Uint32, Uint16);
-  
-public:
-  STATIC_CONST( SignalLength = 4 );
-
-private:
-  Uint32 senderData;
-  Uint32 senderRef;
-  Uint32 tableId;
-  Uint32 tableVersion;
 };
 
 /**
@@ -303,4 +353,7 @@ struct AlterTableRep
   SECTION( TABLE_NAME = 0 );
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
