@@ -1,19 +1,26 @@
-/* Copyright (c) 2003-2007 MySQL AB
-
+/*
+   Copyright (c) 2003, 2010, 2011, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef DICT_TAB_INFO_HPP
 #define DICT_TAB_INFO_HPP
@@ -225,6 +232,10 @@ public:
     Undofile = 23,          ///< Undofile
     HashMap = 24,
 
+    ForeignKey = 25,        // The definition
+    FKParentTrigger = 26,
+    FKChildTrigger = 27,
+
     SchemaTransaction = 30
   };
 
@@ -277,7 +288,9 @@ public:
       tableType == SubscriptionTrigger ||
       tableType == ReadOnlyConstraint ||
       tableType == IndexTrigger ||
-      tableType == ReorgTrigger;
+      tableType == ReorgTrigger ||
+      tableType == FKParentTrigger ||
+      tableType == FKChildTrigger;
   }
   static inline bool
   isFilegroup(int tableType) {
@@ -298,7 +311,12 @@ public:
     return
       tableType == HashMap;
   }
-  
+
+  static inline bool
+  isForeignKey(int tableType) {
+    return tableType == ForeignKey;
+  }
+
   // Object state for translating from/to API
   enum ObjectState {
     StateUndefined = 0,
@@ -431,7 +449,10 @@ public:
     ExtLongvarbinary = NdbSqlUtil::Type::Longvarbinary,
     ExtTime = NdbSqlUtil::Type::Time,
     ExtYear = NdbSqlUtil::Type::Year,
-    ExtTimestamp = NdbSqlUtil::Type::Timestamp
+    ExtTimestamp = NdbSqlUtil::Type::Timestamp,
+    ExtTime2 = NdbSqlUtil::Type::Time2,
+    ExtDatetime2 = NdbSqlUtil::Type::Datetime2,
+    ExtTimestamp2 = NdbSqlUtil::Type::Timestamp2
   };
 
   // Attribute data interpretation
@@ -586,6 +607,22 @@ public:
         AttributeSize = DictTabInfo::an8Bit;
         AttributeArraySize = 4 * AttributeExtLength;
         break;
+      // fractional time types, see wl#946
+      case DictTabInfo::ExtTime2:
+        AttributeSize = DictTabInfo::an8Bit;
+        AttributeArraySize = (3 + (1 + AttributeExtPrecision) / 2)
+                             * AttributeExtLength;
+        break;
+      case DictTabInfo::ExtDatetime2:
+        AttributeSize = DictTabInfo::an8Bit;
+        AttributeArraySize = (5 + (1 + AttributeExtPrecision) / 2)
+                             * AttributeExtLength;
+        break;
+      case DictTabInfo::ExtTimestamp2:
+        AttributeSize = DictTabInfo::an8Bit;
+        AttributeArraySize = (4 + (1 + AttributeExtPrecision) / 2)
+                             * AttributeExtLength;
+        break;
       default:
         return false;
       };
@@ -815,6 +852,86 @@ struct DictHashMapInfo {
   static const SimpleProperties::SP2StructMapping Mapping[];
 };
 
+/**
+ * Foreign Keys
+ */
+struct DictForeignKeyInfo
+{
+  enum KeyValues {
+    ForeignKeyName               = 1,
+    ForeignKeyId                 = 2,
+    ForeignKeyVersion            = 3,
+    ForeignKeyParentTableId      = 4,
+    ForeignKeyParentTableVersion = 5,
+    ForeignKeyChildTableId       = 6,
+    ForeignKeyChildTableVersion  = 7,
+    ForeignKeyParentIndexId      = 8,
+    ForeignKeyParentIndexVersion = 9,
+    ForeignKeyChildIndexId       = 10,
+    ForeignKeyChildIndexVersion  = 11,
+    ForeignKeyOnUpdateAction     = 12,
+    ForeignKeyOnDeleteAction     = 13,
+    ForeignKeyParentTableName    = 14,
+    ForeignKeyParentIndexName    = 15,
+    ForeignKeyChildTableName     = 16,
+    ForeignKeyChildIndexName     = 17,
+    ForeignKeyParentColumnsLength= 18,
+    ForeignKeyParentColumns      = 19,
+    ForeignKeyChildColumnsLength = 20,
+    ForeignKeyChildColumns       = 21
+  };
+
+  // Table data interpretation
+  struct ForeignKey {
+    char   Name[MAX_TAB_NAME_SIZE];
+    char   ParentTableName[MAX_TAB_NAME_SIZE];
+    char   ParentIndexName[MAX_TAB_NAME_SIZE];
+    char   ChildTableName[MAX_TAB_NAME_SIZE];
+    char   ChildIndexName[MAX_TAB_NAME_SIZE];
+    Uint32 ForeignKeyId;
+    Uint32 ForeignKeyVersion;
+    Uint32 ParentTableId;
+    Uint32 ParentTableVersion;
+    Uint32 ChildTableId;
+    Uint32 ChildTableVersion;
+    Uint32 ParentIndexId;
+    Uint32 ParentIndexVersion;
+    Uint32 ChildIndexId;
+    Uint32 ChildIndexVersion;
+    Uint32 OnUpdateAction;
+    Uint32 OnDeleteAction;
+    Uint32 ParentColumnsLength;
+    Uint32 ParentColumns[MAX_ATTRIBUTES_IN_INDEX];
+    Uint32 ChildColumnsLength;
+    Uint32 ChildColumns[MAX_ATTRIBUTES_IN_INDEX];
+    ForeignKey() {}
+    void init();
+  };
+  static const Uint32 MappingSize;
+  static const SimpleProperties::SP2StructMapping Mapping[];
+};
+
+void
+ndbout_print(const DictForeignKeyInfo::ForeignKey& fk, char* buf, size_t sz);
+
+class NdbOut&
+operator<<(class NdbOut& out, const DictForeignKeyInfo::ForeignKey& fk);
+
+#define DFKIMAP(x, y, z) \
+  { DictForeignKeyInfo::y, my_offsetof(x, z), SimpleProperties::Uint32Value, 0, UINT_MAX32, 0 }
+
+#define DFKIMAP2(x, y, z, u, v) \
+  { DictForeignKeyInfo::y, my_offsetof(x, z), SimpleProperties::Uint32Value, u, v, 0 }
+
+#define DFKIMAPS(x, y, z, u, v) \
+  { DictForeignKeyInfo::y, my_offsetof(x, z), SimpleProperties::StringValue, u, v, 0 }
+
+#define DFKIMAPB(x, y, z, u, v, l) \
+  { DictForeignKeyInfo::y, my_offsetof(x, z), SimpleProperties::BinaryValue, u, v, \
+                     my_offsetof(x, l) }
+
+#define DFKIBREAK(x) \
+  { DictForeignKeyInfo::x, 0, SimpleProperties::InvalidValue, 0, 0, 0 }
 
 #undef JAM_FILE_ID
 
