@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -68,6 +68,12 @@ public:
 
     _BackupMin   = 100000,
     BackupStatus = 100000,
+    BackupMinWriteSpeed32 = 100001,
+    BackupMaxWriteSpeed32 = 100002,
+    BackupMaxWriteSpeedOtherNodeRestart32 = 100003,
+    BackupMinWriteSpeed64 = 100004,
+    BackupMaxWriteSpeed64 = 100005,
+    BackupMaxWriteSpeedOtherNodeRestart64 = 100006,
     _BackupMax   = 100999,
 
     _TCMin       = 101000,
@@ -77,6 +83,10 @@ public:
     LQHLogFileInitStatus = 102000,
     _LQHMax = 102999,
 
+    _CMVMIMin = 103000,
+    SetSchedulerResponsiveness = 103000,
+    _CMVMIMax = 103099,
+
     // 1 QMGR Dump information about phase 1 variables
     // 13 CMVMI Dump signal counter
     // 13 NDBCNTR Dump start phase information
@@ -106,6 +116,7 @@ public:
     CmvmiSchedulerSpinTimer = 506,
     // 1222-1225 DICT
     DictDumpLockQueue = 1228,
+    DictDumpGetTabInfoQueue = 1229,
     LqhDumpAllDefinedTabs = 1332,
     LqhDumpNoLogPages = 1333,
     LqhDumpOneScanRec = 2300,
@@ -114,6 +125,7 @@ public:
     LqhDumpLcpState = 2303,
     LqhErrorInsert5042 = 2315,
     LqhDumpPoolLevels = 2353,
+    LqhReportCopyInfo = 2354,
 
     AccDumpOneScanRec = 2400,
     AccDumpAllScanRec = 2401,
@@ -123,18 +135,19 @@ public:
     AccDumpFreeOpRecs = 2405,
     AccDumpNotFreeOpRecs = 2406,
     DumpPageMemory = 1000, // Acc & TUP
-    TcDumpAllScanFragRec = 2500,
+    TcDumpSetOfScanFragRec = 2500,
     TcDumpOneScanFragRec = 2501,
-    TcDumpAllScanRec = 2502,
-    TcDumpAllActiveScanRec = 2503,
+    TcDumpSetOfScanRec = 2502,
     TcDumpOneScanRec = 2504,
     TcDumpOneApiConnectRec = 2505,
-    TcDumpAllApiConnectRec = 2506,
     TcSetTransactionTimeout = 2507,
     TcSetApplTransactionTimeout = 2508,
     TcStartDumpIndexOpCount = 2512,
     TcDumpIndexOpCount = 2513,
     TcDumpApiConnectRecSummary = 2514,
+    TcDumpSetOfApiConnectRec = 2515,
+    TcDumpOneTcConnectRec = 2516,
+    TcDumpSetOfTcConnectRec = 2517,
     TcDumpPoolLevels = 2555,
     CmvmiDumpConnections = 2600,
     CmvmiDumpLongSignalMemory = 2601,
@@ -151,6 +164,9 @@ public:
     CmvmiLongSignalMemorySnapshot = 2608,
     CmvmiLongSignalMemorySnapshotCheck = 2609,
     CmvmiSetKillerWatchdog = 2610,
+    CmvmiLongSignalMemorySnapshotCheck2 = 2611,
+
+    CmvmiShowLongSignalOwnership = 2612, /* Show owners of LSM */
 
     LCPContinue = 5900,
     // 7000 DIH
@@ -194,9 +210,13 @@ public:
     DihSetGcpStopVals = 7026,
     DihStallLcpStart = 7027,
     DihDumpPageRecInfo = 7032,
+    DihFragmentsPerNode = 7033,
+    DihDisplayPauseState = 7034,
     EnableUndoDelayDataWrite = 7080, // DIH+ACC+TUP
     DihSetTimeBetweenGcp = 7090,
     DihStartLcpImmediately = 7099,
+    DihSaveGcpCommitLag = 7100,
+    DihCheckGcpCommitLag = 7101,
     // 8000 Suma
     // 12000 Tux
     TuxLogToFile = 12001,
@@ -225,7 +245,9 @@ public:
     SchemaResourceCheckLeak = 4001, // check same as snapshot
 
     TcResourceSnapshot = 2553,
-    TcResourceCheckLeak = 2554
+    TcResourceCheckLeak = 2554,
+
+    RestoreRates = 30000
   };
 public:
   
