@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -481,15 +481,35 @@ void getTextNR_CopyDict(QQQQ) {
   //-----------------------------------------------------------------------
   // REPORT Node Restart completed copy of dictionary information.
   //-----------------------------------------------------------------------
-  BaseString::snprintf(m_text, m_text_len, 
-		       "Node restart completed copy of dictionary information");
+  if (len == 2)
+  {
+    BaseString::snprintf(m_text, m_text_len, 
+		         "Node restart completed copy of dictionary information"
+                         " to Node %u",
+                         theData[1]);
+  }
+  else
+  {
+    BaseString::snprintf(m_text, m_text_len, 
+		         "Node restart completed copy of dictionary information");
+  }
 }
 void getTextNR_CopyDistr(QQQQ) {
   //-----------------------------------------------------------------------
   // REPORT Node Restart completed copy of distribution information.
   //-----------------------------------------------------------------------
-  BaseString::snprintf(m_text, m_text_len, 
-		       "Node restart completed copy of distribution information");
+  if (len == 2)
+  {
+    BaseString::snprintf(m_text, m_text_len, 
+		   "Node restart completed copy of distribution information"
+                       " to Node %u",
+                       theData[1]);
+  }
+  else
+  {
+    BaseString::snprintf(m_text, m_text_len, 
+		   "Node restart completed copy of distribution information");
+  }
 }
 void getTextNR_CopyFragsStarted(QQQQ) {
   //-----------------------------------------------------------------------
@@ -749,6 +769,14 @@ void getTextStartLog(QQQQ) {
 		       theData[3],
 		       theData[4]);
 }
+void getTextLCPRestored(QQQQ) {
+  //-----------------------------------------------------------------------
+  // REPORT Node Start completed restore of LCP.
+  //-----------------------------------------------------------------------
+  BaseString::snprintf(m_text, m_text_len,
+           "Node Start completed restore of LCP id: %u",
+           theData[1]);
+}
 void getTextStartREDOLog(QQQQ) {
   BaseString::snprintf(m_text, m_text_len, 
 		       "Node: %d StartLog: [GCI Keep: %d LastCompleted: %d NewestRestorable: %d]",
@@ -1428,6 +1456,7 @@ const EventLoggerBase::EventRepLogLevelMatrix EventLoggerBase::matrix[] = {
   ROW(NDBStopCompleted,        LogLevel::llStartUp,     1, Logger::LL_INFO ),
   ROW(NDBStopForced,           LogLevel::llStartUp,     1, Logger::LL_ALERT ),
   ROW(NDBStopAborted,          LogLevel::llStartUp,     1, Logger::LL_INFO ),
+  ROW(LCPRestored,             LogLevel::llStartUp,     7, Logger::LL_INFO ),
   ROW(StartREDOLog,            LogLevel::llStartUp,     4, Logger::LL_INFO ),
   ROW(StartLog,                LogLevel::llStartUp,    10, Logger::LL_INFO ),
   ROW(UNDORecordsExecuted,     LogLevel::llStartUp,    15, Logger::LL_INFO ),
