@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -130,6 +130,7 @@ Dbtup::alloc_var_part(Uint32 * err,
     }
     c_page_pool.getPtr(pagePtr);
     ((Var_page*)pagePtr.p)->init();
+    fragPtr->m_varWordsFree += ((Var_page*)pagePtr.p)->free_space;
     pagePtr.p->list_index = MAX_FREE_LIST - 1;
     LocalDLList<Page> list(c_page_pool, 
 			   fragPtr->free_var_page_array[MAX_FREE_LIST-1]);
@@ -137,10 +138,18 @@ Dbtup::alloc_var_part(Uint32 * err,
   } else {
     c_page_pool.getPtr(pagePtr);
     jam();
-  }
+  }  
+  /*
+    First we remove the current free space on this page from fragment total.
+    Then we calculate a new free space value for the page. Finally we call
+    update_free_page_list() which adds this new value to the fragment total.
+  */
+  ndbassert(fragPtr->m_varWordsFree >= ((Var_page*)pagePtr.p)->free_space);
+  fragPtr->m_varWordsFree -= ((Var_page*)pagePtr.p)->free_space;
   Uint32 idx= ((Var_page*)pagePtr.p)
     ->alloc_record(alloc_size, (Var_page*)ctemp_page, Var_page::CHAIN);
-  
+
+  fragPtr->m_varElemCount++;
   key->m_page_no = pagePtr.i;
   key->m_page_idx = idx;
   
@@ -162,7 +171,11 @@ void Dbtup::free_var_part(Fragrecord* fragPtr,
   if (key->m_page_no != RNIL)
   {
     c_page_pool.getPtr(pagePtr, key->m_page_no);
+    ndbassert(fragPtr->m_varWordsFree >= ((Var_page*)pagePtr.p)->free_space);
+    fragPtr->m_varWordsFree -= ((Var_page*)pagePtr.p)->free_space;
     ((Var_page*)pagePtr.p)->free_record(key->m_page_idx, Var_page::CHAIN);
+    ndbassert(fragPtr->m_varElemCount > 0);
+    fragPtr->m_varElemCount--;
 
     ndbassert(pagePtr.p->free_space <= Var_page::DATA_WORDS);
     if (pagePtr.p->free_space == Var_page::DATA_WORDS - 1)
@@ -175,8 +188,10 @@ void Dbtup::free_var_part(Fragrecord* fragPtr,
       fragPtr->noOfVarPages --;
     } else {
       jam();
+      // Adds the new free space value for the page to the fragment total.
       update_free_page_list(fragPtr, pagePtr);
     }
+    ndbassert(fragPtr->verifyVarSpace());
   }
   return;
 }
@@ -226,7 +241,11 @@ void Dbtup::free_var_rec(Fragrecord* fragPtr,
 void
 Dbtup::free_var_part(Fragrecord* fragPtr, PagePtr pagePtr, Uint32 page_idx)
 {
+  ndbassert(fragPtr->m_varWordsFree >= ((Var_page*)pagePtr.p)->free_space);
+  fragPtr->m_varWordsFree -= ((Var_page*)pagePtr.p)->free_space;
   ((Var_page*)pagePtr.p)->free_record(page_idx, Var_page::CHAIN);
+  ndbassert(fragPtr->m_varElemCount > 0);
+  fragPtr->m_varElemCount--;
 
   ndbassert(pagePtr.p->free_space <= Var_page::DATA_WORDS);
   if (pagePtr.p->free_space == Var_page::DATA_WORDS - 1)
@@ -241,8 +260,10 @@ Dbtup::free_var_part(Fragrecord* fragPtr, PagePtr pagePtr, Uint32 page_idx)
   else
   {
     jam();
+    // Adds the new free space value for the page to the fragment total.
     update_free_page_list(fragPtr, pagePtr);
   }
+  ndbassert(fragPtr->verifyVarSpace());
 }
 
 Uint32 *
@@ -278,13 +299,18 @@ Dbtup::realloc_var_part(Uint32 * err,
       memcpy(copyBuffer, new_var_ptr, 4*oldsz);
       pageP->set_entry_len(oldref.m_page_idx, 0);
       pageP->free_space += oldsz;
+      fragPtr->m_varWordsFree += oldsz;
       pageP->reorg((Var_page*)ctemp_page);
       new_var_ptr= pageP->get_free_space_ptr();
       memcpy(new_var_ptr, copyBuffer, 4*oldsz);
       pageP->set_entry_offset(oldref.m_page_idx, pageP->insert_pos);
       add += oldsz;
     }
+    ndbassert(fragPtr->m_varWordsFree >= pageP->free_space);
+    fragPtr->m_varWordsFree -= pageP->free_space;
+
     pageP->grow_entry(oldref.m_page_idx, add);
+    // Adds the new free space value for the page to the fragment total.
     update_free_page_list(fragPtr, pagePtr);
   }
   else
@@ -357,6 +383,9 @@ Dbtup::move_var_part(Fragrecord* fragPtr, Tablerec* tabPtr, PagePtr pagePtr,
 
   c_page_pool.getPtr(new_pagePtr);
 
+  ndbassert(fragPtr->m_varWordsFree >= ((Var_page*)new_pagePtr.p)->free_space);
+  fragPtr->m_varWordsFree -= ((Var_page*)new_pagePtr.p)->free_space;
+
   Uint32 idx= ((Var_page*)new_pagePtr.p)
     ->alloc_record(size,(Var_page*)ctemp_page, Var_page::CHAIN);
 
@@ -373,8 +402,9 @@ Dbtup::move_var_part(Fragrecord* fragPtr, Tablerec* tabPtr, PagePtr pagePtr,
    */
   memcpy(dst, src, 4*size);
 
+  fragPtr->m_varElemCount++;
   /**
-   * remove old var part of tuple
+   * remove old var part of tuple (and decrement m_varElemCount).
    */
   free_var_part(fragPtr, pagePtr, oldref.m_page_idx);
 
@@ -464,6 +494,9 @@ void Dbtup::update_free_page_list(Fragrecord* fragPtr,
   Uint32 free_space, list_index;
   free_space= pagePtr.p->free_space;
   list_index= pagePtr.p->list_index;
+  fragPtr->m_varWordsFree+= free_space;
+  ndbassert(fragPtr->verifyVarSpace());
+
   if ((free_space < c_min_list_size[list_index]) ||
       (free_space > c_max_list_size[list_index])) {
     Uint32 new_list_index= calculate_free_list_impl(free_space);
