@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003-2006, 2008, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2021, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -19,6 +26,10 @@
 #define DL_HASHTABLE_HPP
 
 #include <ndb_global.h>
+#include "ArrayPool.hpp"
+
+#define JAM_FILE_ID 313
+
 
 /**
  * DLMHashTable implements a hashtable using chaining
@@ -118,13 +129,19 @@ public:
 
   /**
    * Remove element and return to pool
+   * release releases object and places it first in free list
+   * releaseLast releases object and places it last in free list
    */
   void release(Uint32 i);
+  void releaseLast(Uint32 i);
 
   /**
    * Remove element and return to pool
+   * release releases object and places it first in free list
+   * releaseLast releases object and places it last in free list
    */
   void release(Ptr<T> &);
+  void releaseLast(Ptr<T> &);
 
   class Iterator {
   public:
@@ -168,15 +185,8 @@ private:
 template <typename P, typename T, typename M>
 inline
 DLMHashTable<P, T, M>::DLMHashTable(P & _pool)
-  : thePool(_pool)
-{
-  // Require user defined constructor on T since we fiddle
-  // with T's members
-  ASSERT_TYPE_HAS_CONSTRUCTOR(T);
-
-  mask = 0;
-  hashValues = 0;
-}
+  : mask(0), hashValues(NULL), thePool(_pool)
+{}
 
 template <typename P, typename T, typename M>
 inline
@@ -194,20 +204,15 @@ DLMHashTable<P, T, M>::setSize(Uint32 size)
   Uint32 i = 1;
   while (i < size) i *= 2;
 
-  if (mask == (i - 1))
+  if (hashValues != NULL)
   {
-    /**
-     * The size is already set to <b>size</b>
-     */
-    return true;
-  }
-
-  if (mask != 0)
-  {
-    /**
-     * The mask is already set
-     */
-    return false;
+    /*
+      If setSize() is called twice with different size values then this is 
+      most likely a bug.
+    */
+    assert(mask == i-1); 
+    // Return true if size already set to 'size', false otherwise.
+    return mask == i-1;
   }
 
   mask = (i - 1);
@@ -285,7 +290,7 @@ DLMHashTable<P, T, M>::next(Iterator & iter) const
     }
     else
     {
-      iter.curr.i = RNIL;
+      iter.curr.setNull();
       return false;
     }
   }
@@ -305,7 +310,6 @@ DLMHashTable<P, T, M>::remove(Ptr<T> & ptr, const T & key)
   Uint32 i;
   T * p;
   Ptr<T> prev;
-  LINT_INIT(prev.p);
   prev.i = RNIL;
 
   i = hashValues[hv];
@@ -363,6 +367,26 @@ DLMHashTable<P, T, M>::release(Uint32 i)
   release(tmp);
 }
 
+template <typename P, typename T, typename M>
+inline
+void
+DLMHashTable<P, T, M>::releaseLast(Uint32 i)
+{
+  Ptr<T> tmp;
+  tmp.i = i;
+  tmp.p = thePool.getPtr(i);
+  releaseLast(tmp);
+}
+
+template <typename P, typename T, typename M>
+inline
+void
+DLMHashTable<P, T, M>::releaseLast(Ptr<T> & ptr)
+{
+  remove(ptr);
+  thePool.releaseLast(ptr);
+}
+
 template <typename P, typename T, typename M>
 inline
 void
@@ -453,7 +477,7 @@ DLMHashTable<P, T, M>::next(Uint32 bucket, Iterator & iter) const
   if (bucket > mask)
   {
     iter.bucket = bucket;
-    iter.curr.i = RNIL;
+    iter.curr.setNull();
     return false;
   }
 
@@ -529,8 +553,6 @@ DLMHashTable<P, T, M>::find(Ptr<T> & ptr, const T & key) const
 
 // Specializations
 
-#include "ArrayPool.hpp"
-
 template <typename P, typename T, typename U = T >
 class DLHashTableImpl: public DLMHashTable<P, T, DLHashTableDefaultMethods<T, U> >
 {
@@ -551,4 +573,7 @@ private:
   DLHashTable&  operator=(const DLHashTable&);
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
