@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -27,10 +34,12 @@
 #include <GlobalData.hpp>
 #include <TransporterDefinitions.hpp>
 #include <portlib/ndb_prefetch.h>
+#include <portlib/NdbTick.h>
 
 #define JAM_FILE_ID 244
 
 
+#define MAX_SIGNALS_EXECUTED_BEFORE_ZERO_TIME_QUEUE_SCAN 100
 #define MAX_OCCUPANCY 1024
 
 #define JBASIZE   1280 // Jobs which have dead lines to meet use this level
@@ -96,7 +105,7 @@ public:
    FastScheduler();
    ~FastScheduler();
 
-  void doJob();
+  Uint32 doJob(Uint32 loopStartCount);
   void postPoll();
   int checkDoJob();
 
@@ -144,6 +153,10 @@ public:
                               Uint32 *no_exec_loops, Uint32 *tot_exec_time,
                               Uint32 *no_extra_loops, Uint32 *tot_extra_time);
 
+  /* Get/Set high resolution timer in microseconds */
+  NDB_TICKS getHighResTimer() { return curr_ticks; }
+  void setHighResTimer(NDB_TICKS ticks)
+  { curr_ticks = ticks;}
 private:
   void highestAvailablePrio(Priority prio);
   void reportJob(Priority aPriority);
@@ -151,6 +164,7 @@ private:
 
   Uint32 theDoJobTotalCounter;
   Uint32 theDoJobCallCounter;
+  NDB_TICKS curr_ticks;
   Uint8 theJobPriority[4096];
   APZJobBuffer theJobBuffers[JB_LEVELS];
 
@@ -175,7 +189,8 @@ FastScheduler::checkDoJob()
   if (getBOccupancy() < MAX_OCCUPANCY) {
     return 0;
   } else {
-    doJob();
+    Uint32 loopStartCount = 0;
+    doJob(loopStartCount);
     return 1;
   }//if
 }//FastScheduler::checkDoJob()
