@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -74,6 +81,7 @@ template <unsigned T> struct SignalT
     Uint32 theData[T];
     Uint64 dummyAlign;
   };
+  Uint32 m_extra_signals;
 };
 
 /**
@@ -90,6 +98,7 @@ public:
   Uint32 getLength() const;
   Uint32 getTrace() const;
   Uint32 getSendersBlockRef() const;
+  Uint32 getSignalId() const;
 
   const Uint32* getDataPtr() const ;
   Uint32* getDataPtrSend() ;
@@ -104,7 +113,6 @@ public:
   Uint32 length() const { return getLength();}
   BlockReference senderBlockRef() const { return getSendersBlockRef();}
 
-private:
   void setLength(Uint32);
   
 public:
@@ -114,13 +122,18 @@ public:
 #if VMS_DATA_SIZE > 8192
 #error "VMSignal buffer is too small"
 #endif
-  
+
   Uint32 m_sectionPtrI[3];
   SignalHeader header; // 28 bytes
   union {
     Uint32 theData[8192];  // 8192 32-bit words -> 32K Bytes
     Uint64 dummyAlign;
   };
+  /**
+   * A counter used to count extra signals executed as direct signals to ensure we use
+   * proper means for how often to send and flush.
+   */
+  Uint32 m_extra_signals;
   void garbage_register();
 };
 
@@ -161,6 +174,13 @@ Signal::getLength() const {
   return header.theLength;
 }
 
+inline
+Uint32
+Signal::getSignalId() const
+{
+  return header.theSignalId;
+}
+
 inline
 Uint32
 Signal::getTrace() const {
