@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2011, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -18,12 +25,7 @@
 #include "mt_thr_config.hpp"
 #include <kernel/ndb_limits.h>
 #include "../../common/util/parse_mask.hpp"
-
-#ifndef TEST_MT_THR_CONFIG
-#define SUPPORT_CPU_SET 0
-#else
-#define SUPPORT_CPU_SET 1
-#endif
+#include <NdbLockCpuUtil.h>
 
 static const struct THRConfig::Entries m_entries[] =
 {
@@ -33,20 +35,25 @@ static const struct THRConfig::Entries m_entries[] =
   { "recv",  THRConfig::T_RECV,  1, MAX_NDBMT_RECEIVE_THREADS },
   { "rep",   THRConfig::T_REP,   1, 1 },
   { "io",    THRConfig::T_IO,    1, 1 },
+  { "watchdog", THRConfig::T_WD, 1, 1 },
   { "tc",    THRConfig::T_TC,    0, MAX_NDBMT_TC_THREADS },
   { "send",  THRConfig::T_SEND,  0, MAX_NDBMT_SEND_THREADS }
 };
 
 static const struct THRConfig::Param m_params[] =
 {
-  { "count",   THRConfig::Param::S_UNSIGNED },
-  { "cpubind", THRConfig::Param::S_BITMASK },
-  { "cpuset",  THRConfig::Param::S_BITMASK }
+  { "count",    THRConfig::Param::S_UNSIGNED },
+  { "cpubind",  THRConfig::Param::S_BITMASK },
+  { "cpuset",   THRConfig::Param::S_BITMASK },
+  { "realtime", THRConfig::Param::S_UNSIGNED },
+  { "spintime", THRConfig::Param::S_UNSIGNED }
 };
 
 #define IX_COUNT    0
 #define IX_CPUBOUND 1
 #define IX_CPUSET   2
+#define IX_REALTIME 3
+#define IX_SPINTIME 4
 
 static
 unsigned
@@ -116,12 +123,16 @@ THRConfig::setLockIoThreadsToCPU(unsigned val)
 }
 
 void
-THRConfig::add(T_Type t)
+THRConfig::add(T_Type t, unsigned realtime, unsigned spintime)
 {
   T_Thread tmp;
   tmp.m_type = t;
   tmp.m_bind_type = T_Thread::B_UNBOUND;
   tmp.m_no = m_threads[t].size();
+  tmp.m_realtime = realtime;
+  if (spintime > 500)
+    spintime = 500;
+  tmp.m_spintime = spintime;
   m_threads[t].push_back(tmp);
 }
 
@@ -145,18 +156,18 @@ computeThreadConfig(Uint32 MaxNoOfExecutionThreads,
     { 9, 4, 2, 0, 1 },
     { 10, 4, 2, 1, 1 },
     { 11, 4, 3, 1, 1 },
-    { 12, 4, 3, 1, 2 },
-    { 13, 4, 3, 2, 2 },
-    { 14, 4, 4, 2, 2 },
-    { 15, 4, 5, 2, 2 },
+    { 12, 6, 2, 1, 1 },
+    { 13, 6, 3, 1, 1 },
+    { 14, 6, 3, 1, 2 },
+    { 15, 6, 3, 2, 2 },
     { 16, 8, 3, 1, 2 },
     { 17, 8, 4, 1, 2 },
     { 18, 8, 4, 2, 2 },
     { 19, 8, 5, 2, 2 },
-    { 20, 8, 5, 2, 3 },
-    { 21, 8, 5, 3, 3 },
-    { 22, 8, 6, 3, 3 },
-    { 23, 8, 7, 3, 3 },
+    { 20, 10, 4, 2, 2 },
+    { 21, 10, 5, 2, 2 },
+    { 22, 10, 5, 2, 3 },
+    { 23, 10, 6, 2, 3 },
     { 24, 12, 5, 2, 3 },
     { 25, 12, 6, 2, 3 },
     { 26, 12, 6, 3, 3 },
@@ -173,11 +184,39 @@ computeThreadConfig(Uint32 MaxNoOfExecutionThreads,
     { 37, 16, 10, 4, 5 },
     { 38, 16, 11, 4, 5 },
     { 39, 16, 11, 5, 5 },
-    { 40, 16, 12, 5, 5 },
-    { 41, 16, 12, 5, 6 },
-    { 42, 16, 13, 5, 6 },
-    { 43, 16, 13, 6, 6 },
-    { 44, 16, 14, 6, 6 }
+    { 40, 20, 10, 4, 4 },
+    { 41, 20, 10, 4, 5 },
+    { 42, 20, 11, 4, 5 },
+    { 43, 20, 11, 5, 5 },
+    { 44, 20, 12, 5, 5 },
+    { 45, 20, 12, 5, 6 },
+    { 46, 20, 13, 5, 6 },
+    { 47, 20, 13, 6, 6 },
+    { 48, 24, 12, 5, 5 },
+    { 49, 24, 12, 5, 6 },
+    { 50, 24, 13, 5, 6 },
+    { 51, 24, 13, 6, 6 },
+    { 52, 24, 14, 6, 6 },
+    { 53, 24, 14, 6, 7 },
+    { 54, 24, 15, 6, 7 },
+    { 55, 24, 15, 7, 7 },
+    { 56, 24, 16, 7, 7 },
+    { 57, 24, 16, 7, 8 },
+    { 58, 24, 17, 7, 8 },
+    { 59, 24, 17, 8, 8 },
+    { 60, 24, 18, 8, 8 },
+    { 61, 24, 18, 8, 9 },
+    { 62, 24, 19, 8, 9 },
+    { 63, 24, 19, 9, 9 },
+    { 64, 32, 16, 7, 7 },
+    { 65, 32, 16, 7, 8 },
+    { 66, 32, 17, 7, 8 },
+    { 67, 32, 17, 8, 8 },
+    { 68, 32, 18, 8, 8 },
+    { 69, 32, 18, 8, 9 },
+    { 70, 32, 19, 8, 9 },
+    { 71, 32, 20, 8, 9 },
+    { 72, 32, 20, 8, 10 }
   };
 
   Uint32 P = MaxNoOfExecutionThreads - 9;
@@ -195,7 +234,9 @@ computeThreadConfig(Uint32 MaxNoOfExecutionThreads,
 int
 THRConfig::do_parse(unsigned MaxNoOfExecutionThreads,
                     unsigned __ndbmt_lqh_threads,
-                    unsigned __ndbmt_classic)
+                    unsigned __ndbmt_classic,
+                    unsigned realtime,
+                    unsigned spintime)
 {
   /**
    * This is old ndbd.cpp : get_multithreaded_config
@@ -203,9 +244,10 @@ THRConfig::do_parse(unsigned MaxNoOfExecutionThreads,
   if (__ndbmt_classic)
   {
     m_classic = true;
-    add(T_LDM);
-    add(T_MAIN);
-    add(T_IO);
+    add(T_LDM, realtime, spintime);
+    add(T_MAIN, realtime, spintime);
+    add(T_IO, realtime, 0);
+    add(T_WD, realtime, 0);
     const bool allow_too_few_cpus = true;
     return do_bindings(allow_too_few_cpus);
   }
@@ -243,24 +285,25 @@ THRConfig::do_parse(unsigned MaxNoOfExecutionThreads,
     lqhthreads = __ndbmt_lqh_threads;
   }
 
-  add(T_MAIN); /* Global */
-  add(T_REP);  /* Local, main consumer is SUMA */
+  add(T_MAIN, realtime, spintime); /* Global */
+  add(T_REP, realtime, spintime);  /* Local, main consumer is SUMA */
   for(Uint32 i = 0; i < recvthreads; i++)
   {
-    add(T_RECV);
+    add(T_RECV, realtime, spintime);
   }
-  add(T_IO);
+  add(T_IO, realtime, 0);
+  add(T_WD, realtime, 0);
   for(Uint32 i = 0; i < lqhthreads; i++)
   {
-    add(T_LDM);
+    add(T_LDM, realtime, spintime);
   }
   for(Uint32 i = 0; i < tcthreads; i++)
   {
-    add(T_TC);
+    add(T_TC, realtime, spintime);
   }
   for(Uint32 i = 0; i < sendthreads; i++)
   {
-    add(T_SEND);
+    add(T_SEND, realtime, spintime);
   }
 
   // If we have set TC-threads...we say that this is "new" code
@@ -275,16 +318,25 @@ THRConfig::do_parse(unsigned MaxNoOfExecutionThreads,
 int
 THRConfig::do_bindings(bool allow_too_few_cpus)
 {
+  /**
+   * Use LockIoThreadsToCPU also to lock to Watchdog, SocketServer
+   * and SocketClient for backwards compatibility reasons, 
+   * preferred manner is to only use ThreadConfig
+   */
   if (m_LockIoThreadsToCPU.count() == 1)
   {
     m_threads[T_IO][0].m_bind_type = T_Thread::B_CPU_BOUND;
     m_threads[T_IO][0].m_bind_no = m_LockIoThreadsToCPU.getBitNo(0);
+    m_threads[T_WD][0].m_bind_type = T_Thread::B_CPU_BOUND;
+    m_threads[T_WD][0].m_bind_no = m_LockIoThreadsToCPU.getBitNo(0);
   }
   else if (m_LockIoThreadsToCPU.count() > 1)
   {
     unsigned no = createCpuSet(m_LockIoThreadsToCPU);
     m_threads[T_IO][0].m_bind_type = T_Thread::B_CPUSET_BOUND;
     m_threads[T_IO][0].m_bind_no = no;
+    m_threads[T_WD][0].m_bind_type = T_Thread::B_CPUSET_BOUND;
+    m_threads[T_WD][0].m_bind_no = no;
   }
 
   /**
@@ -305,7 +357,7 @@ THRConfig::do_bindings(bool allow_too_few_cpus)
   }
 
   /**
-   * Check that no cpu_sets overlap
+   * Check that no cpu_sets overlap with cpu_bound
    */
   for (unsigned i = 0; i < NDB_ARRAY_SIZE(m_threads); i++)
   {
@@ -343,6 +395,11 @@ THRConfig::do_bindings(bool allow_too_few_cpus)
   unsigned cnt_unbound = 0;
   for (unsigned i = 0; i < NDB_ARRAY_SIZE(m_threads); i++)
   {
+    if (i == T_IO || i == T_WD)
+    {
+      /* IO and Watchdog threads aren't execute threads */
+      continue;
+    }
     for (unsigned j = 0; j < m_threads[i].size(); j++)
     {
       if (m_threads[i][j].m_bind_type == T_Thread::B_CPU_BOUND)
@@ -357,14 +414,6 @@ THRConfig::do_bindings(bool allow_too_few_cpus)
     }
   }
 
-  if (m_threads[T_IO][0].m_bind_type == T_Thread::B_UNBOUND)
-  {
-    /**
-     * don't count this one...
-     */
-    cnt_unbound--;
-  }
-
   if (m_LockExecuteThreadToCPU.count())
   {
     /**
@@ -396,7 +445,7 @@ THRConfig::do_bindings(bool allow_too_few_cpus)
       unsigned no = 0;
       for (unsigned i = 0; i < NDB_ARRAY_SIZE(m_threads); i++)
       {
-        if (i == T_IO)
+        if (i == T_IO || i == T_WD)
           continue;
         for (unsigned j = 0; j < m_threads[i].size(); j++)
         {
@@ -415,7 +464,7 @@ THRConfig::do_bindings(bool allow_too_few_cpus)
       m_info_msg.appfmt("Assigning all threads to CPU %u\n", cpu);
       for (unsigned i = 0; i < NDB_ARRAY_SIZE(m_threads); i++)
       {
-        if (i == T_IO)
+        if (i == T_IO || i == T_WD)
           continue;
         bind_unbound(m_threads[i], cpu);
       }
@@ -537,16 +586,21 @@ THRConfig::do_validate()
   }
 
   /**
-   * LDM can be 1 2 4 8 12 16
+   * LDM can be 1 2 4 6 8 10 12 16 20 24 32
    */
   if (m_threads[T_LDM].size() != 1 &&
       m_threads[T_LDM].size() != 2 &&
       m_threads[T_LDM].size() != 4 &&
+      m_threads[T_LDM].size() != 6 &&
       m_threads[T_LDM].size() != 8 &&
+      m_threads[T_LDM].size() != 10 &&
       m_threads[T_LDM].size() != 12 &&
-      m_threads[T_LDM].size() != 16)
+      m_threads[T_LDM].size() != 16 &&
+      m_threads[T_LDM].size() != 20 &&
+      m_threads[T_LDM].size() != 24 &&
+      m_threads[T_LDM].size() != 32)
   {
-    m_err_msg.assfmt("No of LDM-instances can be 1,2,4,8,12,16. Specified: %u",
+    m_err_msg.assfmt("No of LDM-instances can be 1,2,4,6,8,12,16,24 or 32. Specified: %u",
                      m_threads[T_LDM].size());
     return -1;
   }
@@ -554,61 +608,85 @@ THRConfig::do_validate()
   return 0;
 }
 
+void
+THRConfig::append_name(const char *name,
+                       const char *sep,
+                       bool & append_name_flag)
+{
+  if (!append_name_flag)
+  {
+    m_cfg_string.append(sep);
+    m_cfg_string.append(name);
+    append_name_flag = true;
+  }
+}
+
 const char *
 THRConfig::getConfigString()
 {
   m_cfg_string.clear();
   const char * sep = "";
+  const char * end_sep;
+  const char * start_sep;
+  const char * between_sep;
+  bool append_name_flag;
   for (unsigned i = 0; i < NDB_ARRAY_SIZE(m_threads); i++)
   {
     if (m_threads[i].size())
     {
       const char * name = getEntryName(i);
-      if (i != T_IO)
+      for (unsigned j = 0; j < m_threads[i].size(); j++)
       {
-        for (unsigned j = 0; j < m_threads[i].size(); j++)
+        start_sep = "={";
+        end_sep = "";
+        between_sep="";
+        append_name_flag = false;
+        if (i != T_IO && i != T_WD)
         {
-          m_cfg_string.append(sep);
+          append_name(name, sep, append_name_flag);
           sep=",";
-          m_cfg_string.append(name);
-          if (m_threads[i][j].m_bind_type != T_Thread::B_UNBOUND)
-          {
-            m_cfg_string.append("={");
-            if (m_threads[i][j].m_bind_type == T_Thread::B_CPU_BOUND)
-            {
-              m_cfg_string.appfmt("cpubind=%u", m_threads[i][j].m_bind_no);
-            }
-            else if (m_threads[i][j].m_bind_type == T_Thread::B_CPUSET_BOUND)
-            {
-              m_cfg_string.appfmt("cpuset=%s",
-                                  m_cpu_sets[m_threads[i][j].m_bind_no].str().c_str());
-            }
-            m_cfg_string.append("}");
-          }
         }
-      }
-      else
-      {
-        for (unsigned j = 0; j < m_threads[i].size(); j++)
+        if (m_threads[i][j].m_bind_type != T_Thread::B_UNBOUND)
         {
-          if (m_threads[i][j].m_bind_type != T_Thread::B_UNBOUND)
+          append_name(name, sep, append_name_flag);
+          sep=",";
+          m_cfg_string.append(start_sep);
+          end_sep = "}";
+          start_sep="";
+          if (m_threads[i][j].m_bind_type == T_Thread::B_CPU_BOUND)
           {
-            m_cfg_string.append(sep);
-            sep=",";
-            m_cfg_string.append(name);
-            m_cfg_string.append("={");
-            if (m_threads[i][j].m_bind_type == T_Thread::B_CPU_BOUND)
-            {
-              m_cfg_string.appfmt("cpubind=%u", m_threads[i][j].m_bind_no);
-            }
-            else if (m_threads[i][j].m_bind_type == T_Thread::B_CPUSET_BOUND)
-            {
-              m_cfg_string.appfmt("cpuset=%s",
-                                  m_cpu_sets[m_threads[i][j].m_bind_no].str().c_str());
-            }
-            m_cfg_string.append("}");
+            m_cfg_string.appfmt("cpubind=%u", m_threads[i][j].m_bind_no);
+            between_sep=",";
+          }
+          else if (m_threads[i][j].m_bind_type == T_Thread::B_CPUSET_BOUND)
+          {
+            m_cfg_string.appfmt("cpuset=%s",
+                                m_cpu_sets[m_threads[i][j].m_bind_no].str().c_str());
+            between_sep=",";
           }
         }
+        if (m_threads[i][j].m_spintime || m_threads[i][j].m_realtime)
+        {
+          append_name(name, sep, append_name_flag);
+          sep=",";
+          m_cfg_string.append(start_sep);
+          end_sep = "}";
+          if (m_threads[i][j].m_spintime)
+          {
+            m_cfg_string.append(between_sep);
+            m_cfg_string.appfmt("spintime=%u",
+                                m_threads[i][j].m_spintime);
+            between_sep=",";
+          }
+          if (m_threads[i][j].m_realtime)
+          {
+            m_cfg_string.append(between_sep);
+            m_cfg_string.appfmt("realtime=%u",
+                                m_threads[i][j].m_realtime);
+            between_sep=",";
+          }
+        }
+        m_cfg_string.append(end_sep);
       }
     }
   }
@@ -622,7 +700,7 @@ THRConfig::getThreadCount() const
   Uint32 cnt = 0;
   for (Uint32 i = 0; i < NDB_ARRAY_SIZE(m_threads); i++)
   {
-    if (i != T_IO)
+    if (i != T_IO && i != T_WD)
     {
       cnt += m_threads[i].size();
     }
@@ -711,10 +789,12 @@ parseUnsigned(char *& str, unsigned * dst)
   str = skipblank(str);
   char * endptr = 0;
   errno = 0;
-  long val = strtol(str, &endptr, 0);
-  if (errno == ERANGE)
+  unsigned long val = strtoul(str, &endptr, 0);
+  // Check value out of 'unsigned long' range
+  if (val == ULONG_MAX && errno == ERANGE)
     return -1;
-  if (val < 0 || Int64(val) > 0xFFFFFFFF)
+  // Check value out of 'unsigned' range
+  if (val > UINT_MAX)
     return -1;
   if (endptr == str)
     return -1;
@@ -756,12 +836,6 @@ parseParams(char * str, ParamValue values[], BaseString& err)
     unsigned idx = 0;
     for (; idx < NDB_ARRAY_SIZE(m_params); idx++)
     {
-
-#if ! SUPPORT_CPU_SET
-      if (idx == IX_CPUSET)
-        continue;
-#endif
-
       if (strncasecmp(str, m_params[idx].name, strlen(m_params[idx].name)) == 0)
       {
         str += strlen(m_params[idx].name);
@@ -825,14 +899,20 @@ parseParams(char * str, ParamValue values[], BaseString& err)
 }
 
 int
-THRConfig::find_spec(char *& str, T_Type type)
+THRConfig::find_spec(char *& str,
+                     T_Type type,
+                     unsigned realtime,
+                     unsigned spintime)
 {
   str = skipblank(str);
 
   switch(* str){
   case ',':
   case 0:
-    add(type);
+    if (type != T_IO && type != T_WD)
+      add(type, realtime, spintime);
+    else
+      add(type, realtime, 0);
     return 0;
   }
 
@@ -873,6 +953,8 @@ err:
 
   ParamValue values[NDB_ARRAY_SIZE(m_params)];
   values[IX_COUNT].unsigned_val = 1;
+  values[IX_REALTIME].unsigned_val = realtime;
+  values[IX_SPINTIME].unsigned_val = spintime;
   int res = parseParams(start, values, m_err_msg);
   * end = save;
 
@@ -887,11 +969,20 @@ err:
     return -1;
   }
 
+  if (values[IX_SPINTIME].found &&
+      (type == T_IO || type == T_WD))
+  {
+    m_err_msg.assfmt("Cannot set spintime on IO threads and watchdog threads");
+    return -1;
+  }
+      
   unsigned cnt = values[IX_COUNT].unsigned_val;
   const int index = m_threads[type].size();
   for (unsigned i = 0; i < cnt; i++)
   {
-    add(type);
+    add(type,
+        values[IX_REALTIME].unsigned_val,
+        values[IX_SPINTIME].unsigned_val);
   }
 
   assert(m_threads[type].size() == index + cnt);
@@ -950,7 +1041,9 @@ THRConfig::find_next(char *& str)
 }
 
 int
-THRConfig::do_parse(const char * ThreadConfig)
+THRConfig::do_parse(const char * ThreadConfig,
+                    unsigned realtime,
+                    unsigned spintime)
 {
   BaseString str(ThreadConfig);
   char * ptr = (char*)str.c_str();
@@ -960,7 +1053,7 @@ THRConfig::do_parse(const char * ThreadConfig)
     if (type == T_END)
       return -1;
 
-    if (find_spec(ptr, (T_Type)type) < 0)
+    if (find_spec(ptr, (T_Type)type, realtime, spintime) < 0)
       return -1;
 
     int ret = find_next(ptr);
@@ -974,7 +1067,7 @@ THRConfig::do_parse(const char * ThreadConfig)
   for (Uint32 i = 0; i < T_END; i++)
   {
     while (m_threads[i].size() < m_entries[i].m_min_cnt)
-      add((T_Type)i);
+      add((T_Type)i, realtime, spintime);
   }
 
   const bool allow_too_few_cpus =
@@ -1088,12 +1181,6 @@ THRConfigApplier::getName(const unsigned short list[], unsigned cnt) const
   return getEntryName(thr->m_type);
 }
 
-int
-THRConfigApplier::create_cpusets()
-{
-  return 0;
-}
-
 int
 THRConfigApplier::do_bind(NdbThread* thread,
                           const unsigned short list[], unsigned cnt)
@@ -1109,6 +1196,13 @@ THRConfigApplier::do_bind_io(NdbThread* thread)
   return do_bind(thread, thr);
 }
 
+int
+THRConfigApplier::do_bind_watchdog(NdbThread* thread)
+{
+  const T_Thread* thr = &m_threads[T_WD][0];
+  return do_bind(thread, thr);
+}
+
 int
 THRConfigApplier::do_bind_send(NdbThread* thread, unsigned instance)
 {
@@ -1116,25 +1210,84 @@ THRConfigApplier::do_bind_send(NdbThread* thread, unsigned instance)
   return do_bind(thread, thr);
 }
 
+bool
+THRConfigApplier::do_get_realtime(const unsigned short list[],
+                                  unsigned cnt) const
+{
+  const T_Thread* thr = find_thread(list, cnt);
+  return (bool)thr->m_realtime;
+}
+
+unsigned
+THRConfigApplier::do_get_spintime(const unsigned short list[],
+                                  unsigned cnt) const
+{
+  const T_Thread* thr = find_thread(list, cnt);
+  return thr->m_spintime;
+}
+
+bool
+THRConfigApplier::do_get_realtime_io() const
+{
+  const T_Thread* thr = &m_threads[T_IO][0];
+  return (bool)thr->m_realtime;
+}
+
+bool
+THRConfigApplier::do_get_realtime_wd() const
+{
+  const T_Thread* thr = &m_threads[T_WD][0];
+  return (bool)thr->m_realtime;
+}
+
+bool
+THRConfigApplier::do_get_realtime_send(unsigned instance) const
+{
+  const T_Thread* thr = &m_threads[T_SEND][instance];
+  return (bool)thr->m_realtime;
+}
+
+unsigned
+THRConfigApplier::do_get_spintime_send(unsigned instance) const
+{
+  const T_Thread* thr = &m_threads[T_SEND][instance];
+  return thr->m_spintime;
+}
+
 int
 THRConfigApplier::do_bind(NdbThread* thread,
                           const T_Thread* thr)
 {
+  int res;
   if (thr->m_bind_type == T_Thread::B_CPU_BOUND)
   {
-    int res = NdbThread_LockCPU(thread, thr->m_bind_no);
-    if (res == 0)
-      return 1;
-    else
-      return -res;
+    res = Ndb_LockCPU(thread, thr->m_bind_no);
   }
-#if TODO
   else if (thr->m_bind_type == T_Thread::B_CPUSET_BOUND)
   {
+    SparseBitmask & tmp = m_cpu_sets[thr->m_bind_no];
+    Uint32 num_cpu_ids = tmp.count();
+    Uint32 *cpu_ids = (Uint32*)malloc(sizeof(Uint32) * num_cpu_ids);
+    if (!cpu_ids)
+    {
+      return -errno;
+    }
+    /* Build cpu_ids set from SparseBitmask */
+    for (unsigned i = 0; i < num_cpu_ids; i++)
+    {
+      cpu_ids[i] = tmp.getBitNo(i);
+    }
+    res = Ndb_LockCPUSet(thread, cpu_ids, num_cpu_ids);
+    free((void*)cpu_ids);
   }
-#endif
-
-  return 0;
+  else
+  {
+    return 0;
+  }
+  if (res == 0)
+    return 1;
+  else
+    return -res;
 }
 #endif
 
@@ -1146,7 +1299,7 @@ TAPTEST(mt_thr_config)
 {
   {
     THRConfig tmp;
-    OK(tmp.do_parse(8, 0, 0) == 0);
+    OK(tmp.do_parse(8, 0, 0, 0, 0) == 0);
   }
 
   /**
@@ -1160,7 +1313,11 @@ TAPTEST(mt_thr_config)
         "ldm={cpubind=1-2,5,count=3},ldm",
         "ldm={ cpubind = 1- 2, 5 , count = 3 },ldm",
         "ldm={count=3,cpubind=1-2,5 },  ldm",
-        "ldm={cpuset=1-3,count=3 },ldm",
+        "ldm={cpuset=1-3,count=3,realtime=0,spintime=0 },ldm",
+        "ldm={cpuset=1-3,count=3,realtime=1,spintime=0 },ldm",
+        "ldm={cpuset=1-3,count=3,realtime=0,spintime=1 },ldm",
+        "ldm={cpuset=1-3,count=3,realtime=1,spintime=1 },ldm",
+        "io={cpuset=3,4,6}",
         "main,ldm={},ldm",
         "main,ldm={},ldm,tc",
         "main,ldm={},ldm,tc,tc",
@@ -1176,14 +1333,15 @@ TAPTEST(mt_thr_config)
         "main={ keso=88, count=23},ldm,ldm",
         "main={ cpuset=1-3 }, ldm={cpuset=3-4}",
         "main={ cpuset=1-3 }, ldm={cpubind=2}",
-        "tc,tc,tc",
+        "io={ spintime = 0 }",
+        "tc,tc,tc={count=31}",
         0
       };
 
     for (Uint32 i = 0; ok[i]; i++)
     {
       THRConfig tmp;
-      int res = tmp.do_parse(ok[i]);
+      int res = tmp.do_parse(ok[i], 0, 0);
       printf("do_parse(%s) => %s - %s\n", ok[i],
              res == 0 ? "OK" : "FAIL",
              res == 0 ? "" : tmp.getErrorMessage());
@@ -1191,7 +1349,7 @@ TAPTEST(mt_thr_config)
       {
         BaseString out(tmp.getConfigString());
         THRConfig check;
-        OK(check.do_parse(out.c_str()) == 0);
+        OK(check.do_parse(out.c_str(), 0, 0) == 0);
         OK(strcmp(out.c_str(), check.getConfigString()) == 0);
       }
     }
@@ -1199,7 +1357,7 @@ TAPTEST(mt_thr_config)
     for (Uint32 i = 0; fail[i]; i++)
     {
       THRConfig tmp;
-      int res = tmp.do_parse(fail[i]);
+      int res = tmp.do_parse(fail[i], 0, 0);
       printf("do_parse(%s) => %s - %s\n", fail[i],
              res == 0 ? "OK" : "FAIL",
              res == 0 ? "" : tmp.getErrorMessage());
@@ -1267,7 +1425,7 @@ TAPTEST(mt_thr_config)
     {
       THRConfig tmp;
       tmp.setLockExecuteThreadToCPU(t[i+0]);
-      const int _res = tmp.do_parse(t[i+1]);
+      const int _res = tmp.do_parse(t[i+1], 0, 0);
       const int expect_res = strcmp(t[i+2], "OK") == 0 ? 0 : -1;
       const int res = _res == expect_res ? 0 : -1;
       int ok = expect_res == 0 ?
@@ -1299,11 +1457,10 @@ TAPTEST(mt_thr_config)
 }
 
 #endif
-
 #if 0
 
 /**
- * This c-program was written by mikael ronstrom to
+ * This C-program was written by Mikael Ronstrom to
  *  produce good distribution of threads, given MaxNoOfExecutionThreads
  *
  * Good is based on his experience experimenting/benchmarking
@@ -1416,9 +1573,36 @@ define_num_threads_per_type(Uint32 max_no_exec_threads,
   /* Baseline to start calculations at */
   num_threads[MAIN_THREAD_INDEX] = 1; /* Fixed */
   num_threads[REP_THREAD_INDEX] = 1; /* Fixed */
-  num_lqh_threads = (max_no_exec_threads / 8) * 4;
-  if (num_lqh_threads > 16)
-    num_lqh_threads = 16;
+  num_lqh_threads = (max_no_exec_threads / 4) * 2;
+  if (num_lqh_threads > 32)
+    num_lqh_threads = 32;
+  switch (num_lqh_threads)
+  {
+    case 4:
+    case 6:
+    case 8:
+    case 10:
+    case 12:
+    case 16:
+    case 20:
+    case 24:
+    case 32:
+      break;
+    case 14:
+      num_lqh_threads = 12;
+      break;
+    case 22:
+      num_lqh_threads = 20;
+      break;
+    case 18:
+      num_lqh_threads = 16;
+      break;
+    case 26:
+    case 28:
+    case 30:
+      num_lqh_threads = 24;
+      break;
+  }
   num_threads[LQH_THREAD_INDEX] = num_lqh_threads;
 
   /**
@@ -1455,7 +1639,7 @@ int main(int argc, char *argv)
   Uint32 i;
 
   printf("MaxNoOfExecutionThreads,LQH,TC,send,recv\n");
-  for (i = 9; i < 45; i++)
+  for (i = 9; i <= 72; i++)
   {
     define_num_threads_per_type(i, num_threads);
     printf("{ %u, %u, %u, %u, %u },\n",
@@ -1469,3 +1653,6 @@ int main(int argc, char *argv)
 }
 
 #endif
+
+#define JAM_FILE_ID 297
+
