@@ -1,15 +1,21 @@
 /*
-   Copyright (c) 2007, 2008 MySQL AB, 2008, 2010 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2007, 2021, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -286,8 +292,8 @@ char op_string[] = "aoAO";
 * the six columns' name of test table
 */
 char col_string[] = "ijklmn";
-const int op_len = strlen(op_string);
-const int col_len = strlen(col_string);
+const int op_len = (int)strlen(op_string);
+const int col_len = (int)strlen(col_string);
 
 /*
 * get a random op from "aoAO"
@@ -297,7 +303,7 @@ int get_rand_op_ch(char *ch)
   static unsigned int num = 0;
   if(++num == 0) 
     num = 1;
-  srand(num*time(NULL));
+  srand(num*(unsigned int)time(NULL));
   *ch = op_string[rand() % op_len];
   return 1;
 }
@@ -311,9 +317,9 @@ void change_col_order()
   char temp;
   for (int i = 0; i < 10; i++)  //exchange for 10 times
   {
-    srand(time(NULL)/(i+1));
+    srand((unsigned int)time(NULL)/(i+1));
     pos1 = rand() % col_len;
-    srand((i+1)*time(NULL));
+    srand((i+1)*(unsigned int)time(NULL));
     pos2 = rand() % col_len;
     if (pos1 == pos2)
       continue;
@@ -332,10 +338,10 @@ int get_rand_col_str(char *str)
   static unsigned int num = 0;
   if(++num == 0) 
     num = 1;
-  srand(num*time(NULL));
+  srand(num*(unsigned int)time(NULL));
   len = rand() % col_len + 1;
   change_col_order();
-  snprintf(str, len+1, "%s", col_string);  //len+1, including '\0'
+  BaseString::snprintf(str, len+1, "%s", col_string);  //len+1, including '\0'
   return len;
 }
 
@@ -351,7 +357,8 @@ int get_rand_op_str(char *str)
   len2 = get_rand_col_str(temp+len1);
   len = len1 + len2;
   temp[len] = 'x';
-  snprintf(str, len+1+1, "%s", temp);  //len+1, including '\0'
+  temp[len+1] = '\0'; // Add ending \0
+  BaseString::snprintf(str, len+1+1, "%s", temp);  //len+1, including '\0'
   return len+1;
 }
 
@@ -365,11 +372,11 @@ int get_rand_op_str(char *str)
 int replace_a_to_str(char *source, int pos, char *newstr)
 {
   char temp[MAX_STR_LEN];
-  snprintf(temp, pos+1, "%s", source);
-  snprintf(temp+pos, strlen(newstr)+1, "%s", newstr);
-  snprintf(temp+pos+strlen(newstr), strlen(source)-pos, "%s", source+pos+1);
-  snprintf(source, strlen(temp)+1, "%s", temp); 
-  return strlen(source);
+  BaseString::snprintf(temp, pos+1, "%s", source);
+  BaseString::snprintf(temp+pos, strlen(newstr)+1, "%s", newstr);
+  BaseString::snprintf(temp+pos+strlen(newstr), strlen(source)-pos, "%s", source+pos+1);
+  BaseString::snprintf(source, strlen(temp)+1, "%s", temp);
+  return (int)strlen(source);
 }
 
 /*
@@ -441,7 +448,7 @@ int get_rand_replace_pos(char *str, int len)
   }
   else
   {
-    srand(num*time(NULL));
+    srand(num*(unsigned int)time(NULL));
     pos_col = pos_op + rand() % span + 1;
   }
   return pos_col;
@@ -488,7 +495,7 @@ void get_rand_op_str_compound(char *str)
   if(++num == 0)
     num = 1;
 
-  srand(num*time(NULL));
+  srand(num*(unsigned int)time(NULL));
   level = 1 + rand() % RECURSIVE_LEVEL;
  
   get_rand_op_str(str);
@@ -496,9 +503,9 @@ void get_rand_op_str_compound(char *str)
   for(int i = 0; i < level; i++)
   {
     get_rand_op_str(small_str);
-    tmp = strlen(small_str);
+    tmp = (int)strlen(small_str);
     get_rand_op_str(small_str + tmp);   //get two operations
-    pos = get_rand_replace_pos(str, strlen(str));
+    pos = get_rand_replace_pos(str, (int)strlen(str));
     replace_a_to_str(str, pos, small_str);
   }
 
@@ -676,7 +683,7 @@ void check_all_tuples(char *str, bool *res)
 {    
   for (int i = 0; i < TUPLE_NUM; i++)
   {
-    if(check_one_tuple(i, str, strlen(str)))
+    if(check_one_tuple(i, str, (int)strlen(str)))
       res[i] = true;
   }
 }
@@ -747,13 +754,14 @@ void ndbapi_tuples(Ndb *ndb, char *str, bool *res)
     ERR_EXIT(dict, "Can't get table"TABLE_NAME);
 	
 	const NdbDictionary::Column *col[COL_LEN];
-  for(int i = 0; i < COL_LEN; i++)
+
+  for(int ii = 0; ii < COL_LEN; ii++)
   {
     char tmp[128];
-    col[i] = table->getColumn(COL_NAME[i]);
-	  if(!col[i]) 
+    col[ii] = table->getColumn(COL_NAME[ii]);
+	  if(!col[ii]) 
     {
-      snprintf(tmp, 128, "Can't get column %s", COL_NAME[i]);
+      BaseString::snprintf(tmp, 128, "Can't get column %s", COL_NAME[ii]);
       ERR_EXIT(dict, tmp);
     }
   }
@@ -774,13 +782,13 @@ void ndbapi_tuples(Ndb *ndb, char *str, bool *res)
     ERR_EXIT(scan, "Can't set up read");
   
   NdbRecAttr *rec[COL_LEN];
-  for(int i = 0; i < COL_LEN; i++)
+  for(int ii = 0; ii < COL_LEN; ii++)
   {
     char tmp[128];
-    rec[i] = scan->getValue(COL_NAME[i]);
-	  if(!rec[i]) 
+    rec[ii] = scan->getValue(COL_NAME[ii]);
+	  if(!rec[ii]) 
     {
-      snprintf(tmp, 128, "Can't get rec of %s", COL_NAME[i]);
+      BaseString::snprintf(tmp, 128, "Can't get rec of %s", COL_NAME[ii]);
       ERR_EXIT(scan, tmp);
     }
   }
@@ -949,7 +957,7 @@ int runScanFilterConstructorFail(NDBT_Context* ctx, NDBT_Step* step)
   
   if (trans == NULL)
   {
-    APIERROR(trans->getNdbError());
+    APIERROR(myNdb->getNdbError());
     return NDBT_FAILED;
   }
   
@@ -1015,7 +1023,7 @@ int runScanFilterConstructorFail(NDBT_Context* ctx, NDBT_Step* step)
   
   if (trans == NULL)
   {
-    APIERROR(trans->getNdbError());
+    APIERROR(myNdb->getNdbError());
     return NDBT_FAILED;
   }
   
@@ -1064,9 +1072,10 @@ bool getBit(const Uint32* bitMap,
   return ((bitMap[ bitNum >> 5 ] & (1 << (bitNum & 31))) != 0);
 }
 
-void setBit(Uint32* bitMap,
+void setBit(Uint32* bitMap, int bitMapByteSize,
             int bitNum)
 {
+  require(bitNum >= 0 && bitNum < (bitMapByteSize * 8));
   bitMap[ bitNum >> 5 ] |= (1 << (bitNum & 31));
 }
 
@@ -1286,7 +1295,7 @@ int insertBitRows(Ndb* pNdb)
         if (bitsToSet >= 0)
         {
           for (int idx=0; idx <= bitsToSet; idx++)
-            setBit(buff, idx);
+            setBit(buff, sizeof(buff), idx);
         }
         
         if (insertOp->setValue(col, (char *)buff) != 0)
@@ -1467,18 +1476,18 @@ int verifyBitScanFilter(Ndb* pNdb)
         if ((bitsSetInFilter >= 0) ||
             invert)
         {
-          for (int idx=0; idx <= (32 * buffSize); idx++)
+          for (int idx=0; idx < (32 * buffSize); idx++)
           {
             if ((idx >= offset) && 
                 (idx <= (offset + bitsSetInFilter)))
             {
               if (!invert)
-                setBit(buff, idx);
+                setBit(buff, sizeof(buff), idx);
             }
             else
             {
               if (invert)
-                setBit(buff, idx);
+                setBit(buff, sizeof(buff), idx);
             }
           }
         }
@@ -1671,6 +1680,7 @@ int main(int argc, const char** argv)
   {
     return NDBT_ProgramExit(NDBT_FAILED);
   }
-  
+
+  NDBT_TESTSUITE_INSTANCE(testScanFilter);
   return testScanFilter.execute(argc, argv);
 }
