@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -21,6 +28,7 @@
 #include "NDBT.hpp"
 #include "NDBT_Test.hpp"
 #include <portlib/NdbEnv.h>
+#include <logger/Logger.hpp>
 
 static int opt_stop_on_error = 0;
 
@@ -45,29 +53,29 @@ NDBT_Context::~NDBT_Context(){
 }
 
 const NdbDictionary::Table* NDBT_Context::getTab(){
-  assert(tables.size());
+  require(tables.size());
   return tables[0];
 }
 
 NDBT_TestSuite* NDBT_Context::getSuite(){ 
-  assert(suite != NULL);
+  require(suite != NULL);
   return suite;
 }
 
 NDBT_TestCase* NDBT_Context::getCase(){ 
-  assert(testcase != NULL);
+  require(testcase != NULL);
   return testcase;
 }
 
 const char* NDBT_Context::getTableName(int n) const
 { 
-  assert(suite != NULL);
+  require(suite != NULL);
   return suite->m_tables_in_test[n].c_str();
 }
 
 int NDBT_Context::getNumTables() const
 { 
-  assert(suite != NULL);
+  require(suite != NULL);
   return suite->m_tables_in_test.size();
 }
 
@@ -134,7 +142,7 @@ const char* NDBT_Context::getPropertyWait(const char* _name, const char* _waitVa
 void  NDBT_Context::setProperty(const char* _name, Uint32 _val){ 
   NdbMutex_Lock(propertyMutexPtr);
   const bool b = props.put(_name, _val, true);
-  assert(b == true);
+  require(b == true);
   NdbCondition_Broadcast(propertyCondPtr);
   NdbMutex_Unlock(propertyMutexPtr);
 }
@@ -144,7 +152,7 @@ NDBT_Context::decProperty(const char * name){
   NdbMutex_Lock(propertyMutexPtr);
   Uint32 val = 0;
   if(props.get(name, &val)){
-    assert(val > 0);
+    require(val > 0);
     props.put(name, (val - 1), true);
   }
   NdbCondition_Broadcast(propertyCondPtr);
@@ -179,7 +187,7 @@ NDBT_Context::casProperty(const char * name, Uint32 oldValue, Uint32 newValue)
 void  NDBT_Context::setProperty(const char* _name, const char* _val){ 
   NdbMutex_Lock(propertyMutexPtr);
   const bool b = props.put(_name, _val, true);
-  assert(b == true);
+  require(b == true);
   NdbCondition_Broadcast(propertyCondPtr);
   NdbMutex_Unlock(propertyMutexPtr);
 }
@@ -253,12 +261,12 @@ NDBT_Context::getTables()
 }
 
 void NDBT_Context::setSuite(NDBT_TestSuite* psuite){ 
-  assert(psuite != NULL);
+  require(psuite != NULL);
   suite = psuite;
 }
 
 void NDBT_Context::setCase(NDBT_TestCase* pcase){ 
-  assert(pcase != NULL);
+  require(pcase != NULL);
   testcase = pcase;
 }
 
@@ -321,20 +329,20 @@ NDBT_Step::tearDown(){
 }
 
 Ndb* NDBT_Step::getNdb() const {
-  assert(m_ndb != NULL);
+  require(m_ndb != NULL);
   return m_ndb;
 }
 
 
 int NDBT_Step::execute(NDBT_Context* ctx) {
-  assert(ctx != NULL);
+  require(ctx != NULL);
 
-  int result;  
-
-  g_info << "  |- " << name << " started [" << ctx->suite->getDate() << "]" 
+  char buf[64]; // For timestamp string
+  g_info << "  |- " << name << " started ["
+         << ctx->suite->getDate(buf, sizeof(buf)) << "]"
 	 << endl;
   
-  result = setUp(ctx->m_cluster_connection);
+  int result = setUp(ctx->m_cluster_connection);
   if (result != NDBT_OK){
     return result;
   }
@@ -342,12 +350,23 @@ int NDBT_Step::execute(NDBT_Context* ctx) {
   result = func(ctx, this);
 
   if (result != NDBT_OK) {
-    g_err << "  |- " << name << " FAILED [" << ctx->suite->getDate() 
-	   << "]" << endl;
+    if (result == NDBT_SKIPPED)
+    {
+      g_err << "  |- " << name << " SKIPPED ["
+          << ctx->suite->getDate(buf, sizeof(buf))
+          << "]" << endl;
+    }
+    else
+    {
+      g_err << "  |- " << name << " FAILED ["
+            << ctx->suite->getDate(buf, sizeof(buf))
+            << "]" << endl;
+    }
   }	 
    else {
-    g_info << "  |- " << name << " PASSED [" << ctx->suite->getDate() << "]"
-	   << endl;
+    g_info << "  |- " << name << " PASSED ["
+           << ctx->suite->getDate(buf, sizeof(buf))
+           << "]" << endl;
   }
   
   tearDown();
@@ -356,12 +375,12 @@ int NDBT_Step::execute(NDBT_Context* ctx) {
 }
 
 void NDBT_Step::setContext(NDBT_Context* pctx){
-  assert(pctx != NULL);
+  require(pctx != NULL);
   m_ctx = pctx;
 }
 
 NDBT_Context* NDBT_Step::getContext(){
-  assert(m_ctx != NULL);
+  require(m_ctx != NULL);
   return m_ctx;
 }
 
@@ -394,7 +413,7 @@ NDBT_TestCase::NDBT_TestCase(NDBT_TestSuite* psuite,
   _comment(pcomment),
   suite(psuite)
 {
-  assert(suite != NULL);
+  require(suite != NULL);
 
   m_all_tables = false;
   m_has_run = false;
@@ -441,7 +460,7 @@ NDBT_TestCaseImpl1::~NDBT_TestCaseImpl1(){
 }
 
 int NDBT_TestCaseImpl1::addStep(NDBT_Step* pStep){
-  assert(pStep != NULL);
+  require(pStep != NULL);
   steps.push_back(pStep);
   pStep->setStepNo(steps.size());
   int res = NORESULT;
@@ -450,14 +469,14 @@ int NDBT_TestCaseImpl1::addStep(NDBT_Step* pStep){
 }
 
 int NDBT_TestCaseImpl1::addVerifier(NDBT_Verifier* pVerifier){
-  assert(pVerifier != NULL);
+  require(pVerifier != NULL);
   verifiers.push_back(pVerifier);
   return 0;
 }
 
 int NDBT_TestCaseImpl1::addInitializer(NDBT_Initializer* pInitializer,
                                        bool first){
-  assert(pInitializer != NULL);
+  require(pInitializer != NULL);
   if (first)
     initializers.push(pInitializer, 0);
   else
@@ -466,15 +485,15 @@ int NDBT_TestCaseImpl1::addInitializer(NDBT_Initializer* pInitializer,
 }
 
 int NDBT_TestCaseImpl1::addFinalizer(NDBT_Finalizer* pFinalizer){
-  assert(pFinalizer != NULL);
+  require(pFinalizer != NULL);
   finalizers.push_back(pFinalizer);
   return 0;
 }
 
 void NDBT_TestCaseImpl1::addTable(const char* tableName, bool isVerify) {
-  assert(tableName != NULL);
+  require(tableName != NULL);
   const NdbDictionary::Table* pTable = NDBT_Tables::getTable(tableName);
-  assert(pTable != NULL);
+  require(pTable != NULL);
   testTables.push_back(pTable);
   isVerifyTables = isVerify;
 }
@@ -515,21 +534,21 @@ bool NDBT_TestCaseImpl1::isVerify(const NdbDictionary::Table* aTable) {
 
 void  NDBT_TestCase::setProperty(const char* _name, Uint32 _val){ 
   const bool b = props.put(_name, _val);
-  assert(b == true);
+  require(b == true);
 }
 
 void  NDBT_TestCase::setProperty(const char* _name, const char* _val){ 
   const bool b = props.put(_name, _val);
-  assert(b == true);
+  require(b == true);
 }
 
 
 void *
 runStep(void * s){
-  assert(s != NULL);
+  require(s != NULL);
   NDBT_Step* pStep = (NDBT_Step*)s;
   NDBT_Context* ctx = pStep->getContext();
-  assert(ctx != NULL);
+  require(ctx != NULL);
    // Execute function
   int res = pStep->execute(ctx);
   if(res != NDBT_OK){
@@ -537,7 +556,7 @@ runStep(void * s){
   }
   // Report 
   NDBT_TestCaseImpl1* pCase = (NDBT_TestCaseImpl1*)ctx->getCase();
-  assert(pCase != NULL);
+  require(pCase != NULL);
   pCase->reportStepResult(pStep, res);
   return NULL;
 }
@@ -557,7 +576,7 @@ void NDBT_TestCaseImpl1::startStepInThread(int stepNo, NDBT_Context* ctx){
   char buf[16];
   BaseString::snprintf(buf, sizeof(buf), "step_%d", stepNo);
   Uint32 stackSize = ctx->getProperty(NDBT_TestCase::getStepThreadStackSizePropName(), 
-                                      Uint32(0));
+                                      Uint32(512 * 1024));
 
   NdbThread* pThread = NdbThread_Create(runStep_C,
 					(void**)pStep,
@@ -584,8 +603,8 @@ void NDBT_TestCaseImpl1::waitSteps(){
 	numStepsFail++;
     }       
   }
-  assert(completedSteps == steps.size());
-  assert(completedSteps == numStepsCompleted);
+  require(completedSteps == steps.size());
+  require(completedSteps == numStepsCompleted);
   
   NdbMutex_Unlock(waitThreadsMutexPtr);
   void *status;
@@ -609,7 +628,7 @@ NDBT_TestCaseImpl1::getNoOfCompletedSteps() const {
 
 void NDBT_TestCaseImpl1::reportStepResult(const NDBT_Step* pStep, int result){
   NdbMutex_Lock(waitThreadsMutexPtr);
-  assert(pStep != NULL);
+  require(pStep != NULL);
   for (unsigned i = 0; i < steps.size(); i++){
     if(steps[i] != NULL && steps[i] == pStep){
       results[i] = result;
@@ -623,10 +642,11 @@ void NDBT_TestCaseImpl1::reportStepResult(const NDBT_Step* pStep, int result){
 }
 
 
-int NDBT_TestCase::execute(NDBT_Context* ctx){
-  int res;
-
-  ndbout << "- " << _name << " started [" << ctx->suite->getDate()
+int NDBT_TestCase::execute(NDBT_Context* ctx)
+{
+  char buf[64]; // For timestamp string
+  ndbout << "- " << _name << " started ["
+         << ctx->suite->getDate(buf, sizeof(buf))
 	 << "]" << endl;
 
   ctx->setCase(this);
@@ -637,7 +657,7 @@ int NDBT_TestCase::execute(NDBT_Context* ctx){
 
     PropertiesType pt;
     const bool b = props.getTypeOf(key, &pt);
-    assert(b == true);
+    require(b == true);
     switch(pt){
     case PropertiesType_Uint32:{
       Uint32 val;
@@ -660,6 +680,7 @@ int NDBT_TestCase::execute(NDBT_Context* ctx){
   // test case consist only of initializer
   startTimer(ctx);
   
+  int res;
   if ((res = runInit(ctx)) == NDBT_OK){
     // If initialiser is ok, run steps
     
@@ -678,12 +699,23 @@ int NDBT_TestCase::execute(NDBT_Context* ctx){
   runFinal(ctx); 
 
   if (res == NDBT_OK) {
-    ndbout << "- " << _name << " PASSED [" << ctx->suite->getDate() << "]"
-	   << endl;
+    ndbout << "- " << _name << " PASSED ["
+           << ctx->suite->getDate(buf, sizeof(buf))
+           << "]" << endl;
   }
   else {
-    ndbout << "- " << _name << " FAILED [" << ctx->suite->getDate() << "]"
-	   << endl;
+    if (res == NDBT_SKIPPED)
+    {
+      ndbout << "- " << _name << " SKIPPED ["
+           << ctx->suite->getDate(buf, sizeof(buf))
+           << "]" << endl;
+    }
+    else
+    {
+      ndbout << "- " << _name << " FAILED ["
+             << ctx->suite->getDate(buf, sizeof(buf))
+             << "]" << endl;
+    }
   }
   return res;
 }
@@ -726,9 +758,16 @@ int NDBT_TestCaseImpl1::runSteps(NDBT_Context* ctx){
     startStepInThread(i, ctx);
   waitSteps();
 
+  // Check if any step failed
   for(i = 0; i < steps.size(); i++)
+  {
     if (results[i] != NDBT_OK)
+    {
+      // Found one step which had failed -> report failed
       res = NDBT_FAILED;
+      break;
+    }
+  }
   return res;
 }
 
@@ -778,6 +817,8 @@ void NDBT_TestCaseImpl1::printTestResult(){
       res = "FAILED TO CREATE TABLE";
     else if (tcr->getResult() == FAILED_TO_DISCOVER)
       res = "FAILED TO DISCOVER TABLE";
+    else if (tcr->getResult() == NDBT_SKIPPED)
+      res = "SKIPPED";
     BaseString::snprintf(buf, 255," %-10s %-5s %-20s",
                          tcr->getName(), 
                          res, 
@@ -800,6 +841,7 @@ NDBT_TestSuite::NDBT_TestSuite(const char* pname) :
 {
    numTestsOk = 0;
    numTestsFail = 0;
+   numTestsSkipped = 0;
    numTestsExecuted = 0;
    records = 0;
    loops = 0;
@@ -817,6 +859,10 @@ NDBT_TestSuite::~NDBT_TestSuite(){
     delete tests[i];
   }
   tests.clear();
+  for(unsigned i=0; i<explicitTests.size(); i++){
+    delete explicitTests[i];
+  }
+  explicitTests.clear();
 }
 
 void NDBT_TestSuite::setCreateTable(bool _flag){
@@ -830,7 +876,7 @@ void NDBT_TestSuite::setCreateAllTables(bool _flag){
   m_createAll = _flag;
 }
 void NDBT_TestSuite::setConnectCluster(bool _flag){
-  assert(m_createTable == false);
+  require(m_createTable == false);
   m_connect_cluster = _flag;
 }
 
@@ -859,74 +905,139 @@ bool NDBT_TestSuite::timerIsOn(){
 }
 
 int NDBT_TestSuite::addTest(NDBT_TestCase* pTest){
-  assert(pTest != NULL);
+  require(pTest != NULL);
   tests.push_back(pTest);
   return 0;
 }
 
+int NDBT_TestSuite::addExplicitTest(NDBT_TestCase* pTest){
+  require(pTest != NULL);
+  explicitTests.push_back(pTest);
+  return 0;
+}
+
+NDBT_TestCase*
+NDBT_TestSuite::findTest(const char * testname, bool explicitOK)
+{
+  for (unsigned i = 0; i < tests.size(); i++)
+  {
+    if (strcasecmp(tests[i]->getName(), testname) == 0)
+      return tests[i];
+  }
+
+  if (explicitOK == false)
+    return 0;
+
+  for (unsigned i = 0; i < explicitTests.size(); i++)
+  {
+    if (strcasecmp(explicitTests[i]->getName(), testname) == 0)
+      return explicitTests[i];
+  }
+
+  return 0;
+}
+
 int NDBT_TestSuite::executeAll(Ndb_cluster_connection& con,
 			       const char* _testname){
 
   if(tests.size() == 0)
+  {
+    g_err << "Size of test == 0" << endl;
     return NDBT_FAILED;
+  }
 
-  ndbout << name << " started [" << getDate() << "]" << endl;
+  char buf[64]; // For timestamp string
+  ndbout << name << " started [" << getDate(buf, sizeof(buf)) << "]" << endl;
 
+  testSuiteTimer.doStart();
   if(!runonce)
   {
-    testSuiteTimer.doStart();
     for (int t=0; t < NDBT_Tables::getNumTables(); t++){
       const NdbDictionary::Table* ptab = NDBT_Tables::getTable(t);
       ndbout << "|- " << ptab->getName() << endl;
       execute(con, ptab, _testname);
     }
-    testSuiteTimer.doStop();
   }
   else
   {
-    for (unsigned i = 0; i < tests.size(); i++)
+    NdbDictionary::Table * pTab = 0;
+    if (_testname == NULL)
     {
-      if (opt_stop_on_error != 0 && numTestsFail > 0)
-        break;
+      for (unsigned i = 0; i < tests.size(); i++)
+      {
+        if (opt_stop_on_error != 0 && numTestsFail > 0)
+          break;
 
-      if (_testname != NULL && strcasecmp(tests[i]->getName(), _testname) != 0)
-	continue;
-
-      tests[i]->initBeforeTest();
-      ctx = new NDBT_Context(con);
-
-      ctx->setNumRecords(records);
-      ctx->setNumLoops(loops);
-      ctx->setSuite(this);
-      ctx->setProperty("NoDDL", (Uint32) m_noddl);
-
-      int result = tests[i]->execute(ctx);
-
-      tests[i]->saveTestResult("", result);
-      if (result != NDBT_OK)
-	numTestsFail++;
-      else
-	numTestsOk++;
-      numTestsExecuted++;
-
-      delete ctx;
+        execute(con, tests[i], pTab);
+      }
+    }
+    else
+    {
+      NDBT_TestCase * pt = findTest(_testname);
+      if (pt != 0)
+        execute(con, pt, pTab);
     }
   }
+  testSuiteTimer.doStop();
   return reportAllTables(_testname);
 }
 
-int 
+void
+NDBT_TestSuite::execute(Ndb_cluster_connection& con,
+                        NDBT_TestCase * pTest,
+                        const NdbDictionary::Table *pTab)
+{
+  pTest->initBeforeTest();
+  ctx = new NDBT_Context(con);
+
+  ctx->setNumRecords(records);
+  ctx->setNumLoops(loops);
+  ctx->setSuite(this);
+  ctx->setProperty("NoDDL", (Uint32) m_noddl);
+  if (pTab)
+  {
+    ctx->setTab(pTab);
+  }
+  int result = pTest->execute(ctx);
+  pTest->saveTestResult("", result);
+  if (result != NDBT_OK)
+  {
+    if (result == NDBT_SKIPPED)
+    {
+      numTestsSkipped++;
+    }
+    else
+    {
+      numTestsFail++;
+    }
+  }
+  else
+    numTestsOk++;
+  numTestsExecuted++;
+  delete ctx;
+
+  pTest->m_has_run = true;
+}
+
+int
 NDBT_TestSuite::executeOne(Ndb_cluster_connection& con,
 			   const char* _tabname, const char* _testname){
 
-  if(tests.size() == 0)
+  if (tests.size() == 0 && explicitTests.size() == 0)
+  {
+    g_err << "Test size == 0 and explicit test size == 0" << endl;
     return NDBT_FAILED;
+  }
 
-  ndbout << name << " started [" << getDate() << "]" << endl;
+  char buf[64]; // For timestamp string
+  ndbout << name << " started [" << getDate(buf, sizeof(buf)) << "]" << endl;
 
   const NdbDictionary::Table* ptab = NDBT_Tables::getTable(_tabname);
   if (ptab == NULL)
+  {
+    g_err << "ptab == NULL" << endl;
     return NDBT_FAILED;
+  }
 
   ndbout << "|- " << ptab->getName() << endl;
 
@@ -941,10 +1052,11 @@ NDBT_TestSuite::executeOne(Ndb_cluster_connection& con,
 
 int 
 NDBT_TestSuite::executeOneCtx(Ndb_cluster_connection& con,
-			   const NdbDictionary::Table *ptab, const char* _testname){
-  
-  testSuiteTimer.doStart(); 
-  
+                              const NdbDictionary::Table *ptab,
+                              const char* _testname)
+{
+
+  testSuiteTimer.doStart();
   do{
     if(tests.size() == 0)
       break;
@@ -963,41 +1075,33 @@ NDBT_TestSuite::executeOneCtx(Ndb_cluster_connection& con,
       break;
     }
 
-    ndbout << name << " started [" << getDate() << "]" << endl;
+    char buf[64]; // For timestamp string
+    ndbout << name
+           << " started [" << getDate(buf, sizeof(buf)) << "]" << endl;
     ndbout << "|- " << ptab->getName() << endl;
 
-    for (unsigned t = 0; t < tests.size(); t++)
+    if (_testname == NULL)
     {
-      if (opt_stop_on_error != 0 && numTestsFail > 0)
+      for (unsigned t = 0; t < tests.size(); t++)
+      {
+        if (opt_stop_on_error != 0 && numTestsFail > 0)
+          break;
+
+        execute(con, tests[t], ptab);
+      }
+
+      if (numTestsFail > 0)
         break;
-
-      if (_testname != NULL &&
-          strcasecmp(tests[t]->getName(), _testname) != 0)
-        continue;
-
-      tests[t]->initBeforeTest();
-    
-      ctx = new NDBT_Context(con);
-      ctx->setTab(ptab);
-      ctx->setNumRecords(records);
-      ctx->setNumLoops(loops);
-      ctx->setSuite(this);
-      ctx->setProperty("NoDDL", (Uint32) m_noddl);
-    
-      result = tests[t]->execute(ctx);
-      if (result != NDBT_OK)
-        numTestsFail++;
-      else
-        numTestsOk++;
-      numTestsExecuted++;
-
-      delete ctx;
     }
-
-    if (numTestsFail > 0)
-      break;
-  }while(0);
-
+    else
+    {
+      NDBT_TestCase * pt = findTest(_testname);
+      if (pt != NULL)
+      {
+        execute(con, pt, ptab);
+      }
+    }
+  } while(0);
   testSuiteTimer.doStop();
   int res = report(_testname);
   return NDBT_ProgramExit(res);
@@ -1034,51 +1138,47 @@ NDBT_TestSuite::createHook(Ndb* ndb, NdbDictionary::Table& tab, int when)
 void NDBT_TestSuite::execute(Ndb_cluster_connection& con,
 			     const NdbDictionary::Table* pTab, 
 			     const char* _testname){
-  int result; 
 
-  for (unsigned t = 0; t < tests.size(); t++)
+  if (_testname == NULL)
   {
-    if (opt_stop_on_error != 0 && numTestsFail > 0)
-      break;
-
-    if (_testname != NULL && 
-	strcasecmp(tests[t]->getName(), _testname) != 0)
-      continue;
-
-    if (tests[t]->m_all_tables && tests[t]->m_has_run)
+    for (unsigned t = 0; t < tests.size(); t++)
     {
-      continue;
+      if (opt_stop_on_error != 0 && numTestsFail > 0)
+        break;
+
+      if (tests[t]->m_all_tables && tests[t]->m_has_run)
+      {
+        continue;
+      }
+
+      if (tests[t]->isVerify(pTab) == false) {
+        continue;
+      }
+
+      execute(con, tests[t], pTab);
     }
+  }
+  else
+  {
+    do
+    {
+      NDBT_TestCase * pt = findTest(_testname);
+      if (pt == NULL)
+        break;
 
-    if (tests[t]->isVerify(pTab) == false) {
-      continue;
-    }
+      if (pt->m_all_tables && pt->m_has_run)
+        break;
 
-    tests[t]->initBeforeTest();
+      if (pt->isVerify(pTab) == false)
+        break;
 
-    ctx = new NDBT_Context(con);
-    ctx->setNumRecords(records);
-    ctx->setNumLoops(loops);
-    ctx->setSuite(this);
-    ctx->setTab(pTab);
-    ctx->setProperty("NoDDL", (Uint32) m_noddl);
+      execute(con, pt, pTab);
 
-    result = tests[t]->execute(ctx);
-    tests[t]->saveTestResult(pTab->getName(), result);
-    if (result != NDBT_OK)
-      numTestsFail++;
-    else
-      numTestsOk++;
-    numTestsExecuted++;
-
-    tests[t]->m_has_run = true;
-
-    delete ctx;
+    } while(0);
   }
 }
 
 
-
 int
 NDBT_TestSuite::createTables(Ndb_cluster_connection& con) const
 {
@@ -1237,11 +1337,17 @@ runEmptyDropTable(NDBT_Context* ctx, NDBT_Step* step)
 int 
 NDBT_TestSuite::report(const char* _tcname){
   int result;
-  ndbout << "Completed " << name << " [" << getDate() << "]" << endl;
+  char buf[64]; // For timestamp string
+  ndbout << "Completed " << name
+         << " [" << getDate(buf, sizeof(buf)) << "]" << endl;
   printTestCaseSummary(_tcname);
   ndbout << numTestsExecuted << " test(s) executed" << endl;
   ndbout << numTestsOk << " test(s) OK" 
 	 << endl;
+  if(numTestsSkipped > 0)
+  {
+    ndbout << numTestsSkipped << " test(s) skipped" << endl;
+  }
   if(numTestsFail > 0)
     ndbout << numTestsFail << " test(s) failed"
 	   << endl;
@@ -1249,7 +1355,15 @@ NDBT_TestSuite::report(const char* _tcname){
   if (numTestsFail > 0 || numTestsExecuted == 0){
     result = NDBT_FAILED;
   }else{
-    result = NDBT_OK;
+    if (numTestsSkipped > 0)
+    {
+      /* Any skipped tests summarise run to 'skipped' */
+      result = NDBT_SKIPPED;
+    }
+    else
+    {
+      result = NDBT_OK;
+    }
   }
   return result;
 }
@@ -1263,18 +1377,34 @@ void NDBT_TestSuite::printTestCaseSummary(const char* _tcname){
 
     tests[t]->printTestResult();
   }
+  for (unsigned t = 0; t < explicitTests.size(); t++){
+    if (_tcname != NULL &&
+	strcasecmp(explicitTests[t]->getName(), _tcname) != 0)
+      continue;
+
+    explicitTests[t]->printTestResult();
+  }
   ndbout << "==========================================" << endl;
 }
 
 int NDBT_TestSuite::reportAllTables(const char* _testname){
   int result;
-  ndbout << "Completed running test [" << getDate() << "]" << endl;
+  char buf[64]; // For timestamp string
+  ndbout << "Completed running test ["
+         << getDate(buf, sizeof(buf))
+         << "]" << endl;
   const int totalNumTests = numTestsExecuted;
   printTestCaseSummary(_testname);
   ndbout << numTestsExecuted<< " test(s) executed" << endl;
   ndbout << numTestsOk << " test(s) OK("
 	 <<(int)(((float)numTestsOk/totalNumTests)*100.0) <<"%)" 
 	 << endl;
+  if(numTestsSkipped > 0)
+  {
+    ndbout << numTestsSkipped << " test(s) skipped("
+           <<(int)(((float)numTestsSkipped/totalNumTests)*100.0) <<"%)"
+	   << endl;
+  }
   if(numTestsFail > 0)
     ndbout << numTestsFail << " test(s) failed("
 	   <<(int)(((float)numTestsFail/totalNumTests)*100.0) <<"%)" 
@@ -1284,7 +1414,15 @@ int NDBT_TestSuite::reportAllTables(const char* _testname){
     if (numTestsFail > 0){
       result = NDBT_FAILED;
     }else{
-      result = NDBT_OK;
+      if (numTestsSkipped > 0)
+      {
+        /* Any skipped tests summarise run to 'skipped' */
+        result = NDBT_SKIPPED;
+      }
+      else
+      {
+        result = NDBT_OK;
+      }
     }
   } else {
     result = NDBT_FAILED;
@@ -1399,6 +1537,8 @@ int NDBT_TestSuite::execute(int argc, const char** argv){
   ndb_opt_set_usage_funcs(short_usage_sub, usage);
 
   ndb_load_defaults(NULL, load_default_groups,&argc,&_argv);
+  // Save pointer to memory allocated by 'ndb_load_defaults'
+  char** defaults_argv= _argv;
 
   int ho_error;
 #ifndef DBUG_OFF
@@ -1408,6 +1548,7 @@ int NDBT_TestSuite::execute(int argc, const char** argv){
 			       ndb_std_get_one_option)))
   {
     usage();
+    ndb_free_defaults(defaults_argv);
     return NDBT_ProgramExit(NDBT_WRONGARGS);
   }
 
@@ -1486,28 +1627,71 @@ int NDBT_TestSuite::execute(int argc, const char** argv){
                            *dropFunc);
       pt->addFinalizer(ptf);
     }
+
+    for (unsigned t = 0; t < explicitTests.size(); t++)
+    {
+      const char* createFuncName= NULL;
+      NDBT_TESTFUNC* createFunc= NULL;
+      const char* dropFuncName= NULL;
+      NDBT_TESTFUNC* dropFunc= NULL;
+
+      if (!m_noddl)
+      {
+        createFuncName= m_createAll ? "runCreateTable" : "runCreateTable";
+        createFunc=   m_createAll ? &runCreateTables : &runCreateTable;
+        dropFuncName= m_createAll ? "runDropTables" : "runDropTable";
+        dropFunc= m_createAll ? &runDropTables : &runDropTable;
+      }
+      else
+      {
+        /* No DDL allowed, so we substitute 'do nothing' variants
+         * of the create + drop table test procs
+         */
+        createFuncName= "runCheckTableExists";
+        createFunc= &runCheckTableExists;
+        dropFuncName= "runEmptyDropTable";
+        dropFunc= &runEmptyDropTable;
+      }
+
+      NDBT_TestCaseImpl1* pt= (NDBT_TestCaseImpl1*)explicitTests[t];
+      NDBT_Initializer* pti =
+        new NDBT_Initializer(pt,
+                             createFuncName,
+                             *createFunc);
+      pt->addInitializer(pti, true);
+      NDBT_Finalizer* ptf =
+        new NDBT_Finalizer(pt,
+                           dropFuncName,
+                           *dropFunc);
+      pt->addFinalizer(ptf);
+    }
   }
 
   if (opt_print == true){
     printExecutionTree();
+    ndb_free_defaults(defaults_argv);
     return 0;
   }
 
   if (opt_print_html == true){
     printExecutionTreeHTML();
+    ndb_free_defaults(defaults_argv);
     return 0;
   }
 
   if (opt_print_cases == true){
     printCases();
+    ndb_free_defaults(defaults_argv);
     return 0;
   }
 
   Ndb_cluster_connection con(opt_ndb_connectstring, opt_ndb_nodeid);
   if(m_connect_cluster && con.connect(12, 5, 1))
   {
+    ndb_free_defaults(defaults_argv);
     return NDBT_ProgramExit(NDBT_FAILED);
   }
+  con.set_optimized_node_selection(opt_ndb_optimized_node_selection);
 
   if(argc == 0){
     // No table specified
@@ -1521,6 +1705,7 @@ int NDBT_TestSuite::execute(int argc, const char** argv){
     res = report(opt_testname);
   }
 
+  ndb_free_defaults(defaults_argv);
   return NDBT_ProgramExit(res);
 }
 
@@ -1531,6 +1716,10 @@ void NDBT_TestSuite::printExecutionTree(){
     tests[t]->print();
     ndbout << endl;
   } 
+  for (unsigned t = 0; t < explicitTests.size(); t++){
+    explicitTests[t]->print();
+    ndbout << endl;
+  } 
 }
 
 void NDBT_TestSuite::printExecutionTreeHTML(){
@@ -1541,7 +1730,10 @@ void NDBT_TestSuite::printExecutionTreeHTML(){
     tests[t]->printHTML();
     ndbout << endl;
   } 
-
+  for (unsigned t = 0; t < explicitTests.size(); t++){
+    explicitTests[t]->printHTML();
+    ndbout << endl;
+  }
 }
 
 void NDBT_TestSuite::printCases(){
@@ -1550,29 +1742,22 @@ void NDBT_TestSuite::printCases(){
   for (unsigned t = 0; t < tests.size(); t++){
     ndbout << name << " -n " << tests[t]->getName() << endl;
   } 
+  ndbout << "# Number of explicit tests: " << explicitTests.size() << endl;
+  for (unsigned t = 0; t < explicitTests.size(); t++){
+    ndbout << name << " -n " << explicitTests[t]->getName() << endl;
+  } 
 }
 
-const char* NDBT_TestSuite::getDate(){
-  static char theTime[128];
-  struct tm* tm_now;
+const char* NDBT_TestSuite::getDate(char* str, size_t len)
+{
+  // Get current time
   time_t now;
-  now = time((time_t*)NULL);
-#ifdef NDB_WIN32
-  tm_now = localtime(&now);
-#else
-  tm_now = gmtime(&now);
-#endif
-  
-  BaseString::snprintf(theTime, 128,
-	   "%d-%.2d-%.2d %.2d:%.2d:%.2d",
-	   tm_now->tm_year + 1900, 
-	   tm_now->tm_mon + 1, 
-	   tm_now->tm_mday,
-	   tm_now->tm_hour,
-	   tm_now->tm_min,
-	   tm_now->tm_sec);
+  time(&now);
 
-  return theTime;
+  // Print as timestamp to buf
+  Logger::format_timestamp(now, str, len);
+
+  return str;
 }
 
 void NDBT_TestCaseImpl1::printHTML(){
@@ -1592,7 +1777,7 @@ void NDBT_TestCaseImpl1::print(){
   for(const char * key = it.first(); key != 0; key = it.next()){
     PropertiesType pt;
     const bool b = props.getTypeOf(key, &pt);
-    assert(b == true);
+    require(b == true);
     switch(pt){
     case PropertiesType_Uint32:{
       Uint32 val;
@@ -1685,6 +1870,12 @@ NDBT_Context::closeToTimeout(int safety)
   return false;
 }
 
+NdbApiConfig const&
+NDBT_Context::getConfig() const
+{
+  return m_cluster_connection.m_impl.m_config;
+}
+
 template class Vector<NDBT_TestCase*>;
 template class Vector<NDBT_TestCaseResult*>;
 template class Vector<NDBT_Step*>;
